#!/usr/bin/env python

#
# Generated Sat Mar  8 16:15:12 2014 by parse_xsd.py version 0.5.
#

import saml2
from saml2 import SamlBase


NAMESPACE = "urn:oasis:names:tc:SAML:metadata:algsupport"


class DigestMethodType_(SamlBase):
    """The urn:oasis:names:tc:SAML:metadata:algsupport:DigestMethodType
    element"""

    c_tag = "DigestMethodType"
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_attributes["Algorithm"] = ("algorithm", "anyURI", True)

    def __init__(self, algorithm=None, text=None, extension_elements=None, extension_attributes=None):
        SamlBase.__init__(
            self, text=text, extension_elements=extension_elements, extension_attributes=extension_attributes
        )
        self.algorithm = algorithm


def digest_method_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(DigestMethodType_, xml_string)


class SigningMethodType_(SamlBase):
    """The urn:oasis:names:tc:SAML:metadata:algsupport:SigningMethodType
    element"""

    c_tag = "SigningMethodType"
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_attributes["Algorithm"] = ("algorithm", "anyURI", True)
    c_attributes["MinKeySize"] = ("min_key_size", "positiveInteger", False)
    c_attributes["MaxKeySize"] = ("max_key_size", "positiveInteger", False)

    def __init__(
        self,
        algorithm=None,
        min_key_size=None,
        max_key_size=None,
        text=None,
        extension_elements=None,
        extension_attributes=None,
    ):
        SamlBase.__init__(
            self, text=text, extension_elements=extension_elements, extension_attributes=extension_attributes
        )
        self.algorithm = algorithm
        self.min_key_size = min_key_size
        self.max_key_size = max_key_size


def signing_method_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(SigningMethodType_, xml_string)


class DigestMethod(DigestMethodType_):
    """The urn:oasis:names:tc:SAML:metadata:algsupport:DigestMethod element"""

    c_tag = "DigestMethod"
    c_namespace = NAMESPACE
    c_children = DigestMethodType_.c_children.copy()
    c_attributes = DigestMethodType_.c_attributes.copy()
    c_child_order = DigestMethodType_.c_child_order[:]
    c_cardinality = DigestMethodType_.c_cardinality.copy()


def digest_method_from_string(xml_string):
    return saml2.create_class_from_xml_string(DigestMethod, xml_string)


class SigningMethod(SigningMethodType_):
    """The urn:oasis:names:tc:SAML:metadata:algsupport:SigningMethod element"""

    c_tag = "SigningMethod"
    c_namespace = NAMESPACE
    c_children = SigningMethodType_.c_children.copy()
    c_attributes = SigningMethodType_.c_attributes.copy()
    c_child_order = SigningMethodType_.c_child_order[:]
    c_cardinality = SigningMethodType_.c_cardinality.copy()


def signing_method_from_string(xml_string):
    return saml2.create_class_from_xml_string(SigningMethod, xml_string)


ELEMENT_FROM_STRING = {
    DigestMethod.c_tag: digest_method_from_string,
    DigestMethodType_.c_tag: digest_method_type__from_string,
    SigningMethod.c_tag: signing_method_from_string,
    SigningMethodType_.c_tag: signing_method_type__from_string,
}

ELEMENT_BY_TAG = {
    "DigestMethod": DigestMethod,
    "DigestMethodType": DigestMethodType_,
    "SigningMethod": SigningMethod,
    "SigningMethodType": SigningMethodType_,
}


def factory(tag, **kwargs):
    return ELEMENT_BY_TAG[tag](**kwargs)



EDUCOURSE_OID = "urn:oid:1.3.6.1.4.1.5923.1.6.1."
EDUPERSON_OID = "urn:oid:1.3.6.1.4.1.5923.1.1.1."
EDUMEMBER1_OID = "urn:oid:1.3.6.1.4.1.5923.1.5.1."

# voPerson class References:
# - https://github.com/voperson/voperson/blob/2.0.0/voPerson.md#voperson-object-class-definition
VOPERSON_V2_OID = "urn:oid:1.3.6.1.4.1.25178.4.1."

# ldap.gv.at definitions as specified in:
# http://www.ref.gv.at/AG-IZ-PVP2-Version-2-1-0-2.2754.0.html
LDAPGVAT_OID = "urn:oid:1.2.40.0.10.2.1.1."

UCL_DIR_PILOT = "urn:oid:0.9.2342.19200300.100.1."
X500ATTR_OID = "urn:oid:2.5.4."
LDAPGVAT_UCL_DIR_PILOT = UCL_DIR_PILOT
LDAPGVAT_X500ATTR_OID = X500ATTR_OID
NETSCAPE_LDAP = "urn:oid:2.16.840.1.113730.3.1."
NOREDUPERSON_OID = "urn:oid:1.3.6.1.4.1.2428.90.1."
PKCS_9 = "urn:oid:1.2.840.113549.1.9.1."
SCHAC = "urn:oid:1.3.6.1.4.1.25178.1.2."
SIS = "urn:oid:1.2.752.194.10.2."
UMICH = "urn:oid:1.3.6.1.4.1.250.1.57."

# openosi-0.82.schema http://www.openosi.org/osi/display/ldap/Home
OPENOSI_OID = "urn:oid:1.3.6.1.4.1.27630.2.1.1."
SWISSEDUPERSON_OID = "urn:oid:2.16.756.1.2.5.1.1."

EIDAS_NATURALPERSON = "http://eidas.europa.eu/attributes/naturalperson/"
EIDAS_LEGALPERSON = "http://eidas.europa.eu/attributes/legalperson/"

# SAML subject id specification
# https://docs.oasis-open.org/security/saml-subject-id-attr/v1.0/cs01/saml-subject-id-attr-v1.0-cs01.html
SAML_SUBJECT_ID = "urn:oasis:names:tc:SAML:attribute:"

# umbrellaID specification - https://www.umbrellaid.org
# https://github.com/Umbrella-Commiters/UmbrellaIdP3/blob/master/schema/99-user.ldif
UMBRELLA_EAAUser_ID = "urn:oid:1.3.6.1.4.1.42750.1.1."

# PKIX specification (SMI Security for PKIX Personal Data Attributes)
# https://tools.ietf.org/html/rfc7299
PKIX_OID = "urn:oid:1.3.6.1.5.5.7.9."

# INERA specification
# Closet public spec source I could find, sadly in swedish
# https://www.sambi.se/wordpress/wp-content/uploads/2017/06/Sambi_Attributspecifikation_1.1.pdf
INERA_OID = "urn:oid:1.2.752.29.4."

# DIGG specification
# https://docs.swedenconnect.se/technical-framework/latest/ELN-0604_-_Attribute_Specification_for_the_Swedish_eID_Framework.html
DIGG_OID = "urn:oid:1.2.752.201.3."

MAP = {
    "identifier": "urn:oasis:names:tc:SAML:2.0:attrname-format:uri",
    "fro": {
        f"{DIGG_OID}1": "orgAffiliation",
        f"{DIGG_OID}2": "transactionIdentifier",
        f"{DIGG_OID}3": "authContextParams",
        f"{DIGG_OID}4": "prid",
        f"{DIGG_OID}5": "pridPersistence",
        f"{DIGG_OID}6": "personalIdentityNumberBinding",
        f"{DIGG_OID}7": "eidasPersonIdentifier",
        f"{DIGG_OID}8": "birthName",
        f"{DIGG_OID}9": "eidasNaturalPersonAddress",
        f"{DIGG_OID}10": "userCertificate",
        f"{DIGG_OID}11": "userSignature",
        f"{DIGG_OID}12": "sad",
        f"{DIGG_OID}13": "authServerSignature",
        f"{DIGG_OID}14": "signMessageDigest",
        f"{EIDAS_LEGALPERSON}LegalPersonIdentifier": "LegalPersonIdentifier",
        f"{EIDAS_LEGALPERSON}LegalPersonAddress": "LegalAddress",
        f"{EIDAS_LEGALPERSON}LegalName": "LegalName",
        f"{EIDAS_LEGALPERSON}VATRegistrationNumber": "VATRegistration",
        f"{EIDAS_LEGALPERSON}TaxReference": "TaxReference",
        f"{EIDAS_LEGALPERSON}BusinessCodes": "BusinessCodes",
        f"{EIDAS_LEGALPERSON}LEI": "LEI",
        f"{EIDAS_LEGALPERSON}EORI": "EORI",
        f"{EIDAS_LEGALPERSON}SEED": "SEED",
        f"{EIDAS_LEGALPERSON}SIC": "SIC",
        f"{EIDAS_LEGALPERSON}D-2012-17-EUIdentifier": "D-2012-17-EUIdentifier",
        f"{EIDAS_NATURALPERSON}PersonIdentifier": "PersonIdentifier",
        f"{EIDAS_NATURALPERSON}CurrentFamilyName": "FamilyName",
        f"{EIDAS_NATURALPERSON}CurrentGivenName": "FirstName",
        f"{EIDAS_NATURALPERSON}DateOfBirth": "DateOfBirth",
        f"{EIDAS_NATURALPERSON}BirthName": "BirthName",
        f"{EIDAS_NATURALPERSON}PlaceOfBirth": "PlaceOfBirth",
        f"{EIDAS_NATURALPERSON}CurrentAddress": "CurrentAddress",
        f"{EIDAS_NATURALPERSON}Gender": "Gender",
        f"{EDUCOURSE_OID}1": "eduCourseOffering",
        f"{EDUCOURSE_OID}2": "eduCourseMember",
        f"{EDUMEMBER1_OID}1": "isMemberOf",
        f"{EDUPERSON_OID}1": "eduPersonAffiliation",
        f"{EDUPERSON_OID}2": "eduPersonNickname",
        f"{EDUPERSON_OID}3": "eduPersonOrgDN",
        f"{EDUPERSON_OID}4": "eduPersonOrgUnitDN",
        f"{EDUPERSON_OID}5": "eduPersonPrimaryAffiliation",
        f"{EDUPERSON_OID}6": "eduPersonPrincipalName",
        f"{EDUPERSON_OID}7": "eduPersonEntitlement",
        f"{EDUPERSON_OID}8": "eduPersonPrimaryOrgUnitDN",
        f"{EDUPERSON_OID}9": "eduPersonScopedAffiliation",
        f"{EDUPERSON_OID}10": "eduPersonTargetedID",
        f"{EDUPERSON_OID}11": "eduPersonAssurance",
        f"{EDUPERSON_OID}12": "eduPersonPrincipalNamePrior",
        f"{EDUPERSON_OID}13": "eduPersonUniqueId",
        f"{EDUPERSON_OID}16": "eduPersonOrcid",
        f"{INERA_OID}1": "employeeHsaId",
        f"{INERA_OID}13": "personalIdentityNumber",
        f"{LDAPGVAT_OID}1": "PVP-GID",
        f"{LDAPGVAT_OID}149": "PVP-BPK",
        f"{LDAPGVAT_OID}153": "PVP-OU-OKZ",
        f"{LDAPGVAT_OID}261.10": "PVP-VERSION",
        f"{LDAPGVAT_OID}261.20": "PVP-PRINCIPAL-NAME",
        f"{LDAPGVAT_OID}261.24": "PVP-PARTICIPANT-OKZ",
        f"{LDAPGVAT_OID}261.30": "PVP-ROLES",
        f"{LDAPGVAT_OID}261.40": "PVP-INVOICE-RECPT-ID",
        f"{LDAPGVAT_OID}261.50": "PVP-COST-CENTER-ID",
        f"{LDAPGVAT_OID}261.60": "PVP-CHARGE-CODE",
        f"{LDAPGVAT_OID}3": "PVP-OU-GV-OU-ID",
        f"{LDAPGVAT_OID}33": "PVP-FUNCTION",
        f"{LDAPGVAT_OID}55": "PVP-BIRTHDATE",
        f"{LDAPGVAT_OID}71": "PVP-PARTICIPANT-ID",
        f"{LDAPGVAT_UCL_DIR_PILOT}1": "PVP-USERID",
        f"{LDAPGVAT_UCL_DIR_PILOT}3": "PVP-MAIL",
        f"{LDAPGVAT_X500ATTR_OID}11": "PVP-OU",
        f"{LDAPGVAT_X500ATTR_OID}20": "PVP-TEL",
        f"{LDAPGVAT_X500ATTR_OID}42": "PVP-GIVENNAME",
        f"{NETSCAPE_LDAP}1": "carLicense",
        f"{NETSCAPE_LDAP}2": "departmentNumber",
        f"{NETSCAPE_LDAP}3": "employeeNumber",
        f"{NETSCAPE_LDAP}4": "employeeType",
        f"{NETSCAPE_LDAP}13": "mailLocalAddress",
        f"{NETSCAPE_LDAP}39": "preferredLanguage",
        f"{NETSCAPE_LDAP}40": "userSMIMECertificate",
        f"{NETSCAPE_LDAP}216": "userPKCS12",
        f"{NETSCAPE_LDAP}241": "displayName",
        f"{NOREDUPERSON_OID}1": "norEduOrgUniqueNumber",
        f"{NOREDUPERSON_OID}2": "norEduOrgUnitUniqueNumber",
        f"{NOREDUPERSON_OID}3": "norEduPersonBirthDate",
        f"{NOREDUPERSON_OID}4": "norEduPersonLIN",
        f"{NOREDUPERSON_OID}5": "norEduPersonNIN",
        f"{NOREDUPERSON_OID}6": "norEduOrgAcronym",
        f"{NOREDUPERSON_OID}7": "norEduOrgUniqueIdentifier",
        f"{NOREDUPERSON_OID}8": "norEduOrgUnitUniqueIdentifier",
        f"{NOREDUPERSON_OID}9": "federationFeideSchemaVersion",
        f"{NOREDUPERSON_OID}10": "norEduPersonLegalName",
        f"{NOREDUPERSON_OID}11": "norEduOrgSchemaVersion",
        f"{NOREDUPERSON_OID}12": "norEduOrgNIN",
        f"{OPENOSI_OID}17": "osiHomeUrl",
        f"{OPENOSI_OID}19": "osiPreferredTZ",
        f"{OPENOSI_OID}72": "osiICardTimeLastUpdated",
        f"{OPENOSI_OID}104": "osiMiddleName",
        f"{OPENOSI_OID}107": "osiOtherEmail",
        f"{OPENOSI_OID}109": "osiOtherHomePhone",
        f"{OPENOSI_OID}120": "osiWorkURL",
        f"{PKCS_9}1": "email",
        f"{PKIX_OID}1": "dateOfBirth",
        f"{PKIX_OID}2": "placeOfBirth",
        f"{PKIX_OID}3": "gender",
        f"{PKIX_OID}4": "countryOfCitizenship",
        f"{PKIX_OID}5": "countryOfResidence",
        f"{SAML_SUBJECT_ID}subject-id": "subject-id",
        f"{SAML_SUBJECT_ID}pairwise-id": "pairwise-id",
        f"{SCHAC}1": "schacMotherTongue",
        f"{SCHAC}2": "schacGender",
        f"{SCHAC}3": "schacDateOfBirth",
        f"{SCHAC}4": "schacPlaceOfBirth",
        f"{SCHAC}5": "schacCountryOfCitizenship",
        f"{SCHAC}6": "schacSn1",
        f"{SCHAC}7": "schacSn2",
        f"{SCHAC}8": "schacPersonalTitle",
        f"{SCHAC}9": "schacHomeOrganization",
        f"{SCHAC}10": "schacHomeOrganizationType",
        f"{SCHAC}11": "schacCountryOfResidence",
        f"{SCHAC}12": "schacUserPresenceID",
        f"{SCHAC}13": "schacPersonalPosition",
        f"{SCHAC}14": "schacPersonalUniqueCode",
        f"{SCHAC}15": "schacPersonalUniqueID",
        f"{SCHAC}17": "schacExpiryDate",
        f"{SCHAC}18": "schacUserPrivateAttribute",
        f"{SCHAC}19": "schacUserStatus",
        f"{SCHAC}20": "schacProjectMembership",
        f"{SCHAC}21": "schacProjectSpecificRole",
        f"{SIS}1": "sisLegalGuardianFor",
        f"{SIS}2": "sisSchoolGrade",
        f"{UCL_DIR_PILOT}1": "uid",
        f"{UCL_DIR_PILOT}3": "mail",
        f"{UCL_DIR_PILOT}25": "dc",
        f"{UCL_DIR_PILOT}37": "associatedDomain",
        f"{UCL_DIR_PILOT}43": "co",
        f"{UCL_DIR_PILOT}60": "jpegPhoto",
        f"{UMBRELLA_EAAUser_ID}1": "EAAHash",
        f"{UMBRELLA_EAAUser_ID}3": "EAAKey",
        f"{UMICH}57": "labeledURI",
        f"{X500ATTR_OID}2": "knowledgeInformation",
        f"{X500ATTR_OID}3": "cn",
        f"{X500ATTR_OID}4": "sn",
        f"{X500ATTR_OID}5": "serialNumber",
        f"{X500ATTR_OID}6": "c",
        f"{X500ATTR_OID}7": "l",
        f"{X500ATTR_OID}8": "st",
        f"{X500ATTR_OID}9": "street",
        f"{X500ATTR_OID}10": "o",
        f"{X500ATTR_OID}11": "ou",
        f"{X500ATTR_OID}12": "title",
        f"{X500ATTR_OID}14": "searchGuide",
        f"{X500ATTR_OID}15": "businessCategory",
        f"{X500ATTR_OID}16": "postalAddress",
        f"{X500ATTR_OID}17": "postalCode",
        f"{X500ATTR_OID}18": "postOfficeBox",
        f"{X500ATTR_OID}19": "physicalDeliveryOfficeName",
        f"{X500ATTR_OID}20": "telephoneNumber",
        f"{X500ATTR_OID}21": "telexNumber",
        f"{X500ATTR_OID}22": "teletexTerminalIdentifier",
        f"{X500ATTR_OID}23": "facsimileTelephoneNumber",
        f"{X500ATTR_OID}24": "x121Address",
        f"{X500ATTR_OID}25": "internationaliSDNNumber",
        f"{X500ATTR_OID}26": "registeredAddress",
        f"{X500ATTR_OID}27": "destinationIndicator",
        f"{X500ATTR_OID}28": "preferredDeliveryMethod",
        f"{X500ATTR_OID}29": "presentationAddress",
        f"{X500ATTR_OID}30": "supportedApplicationContext",
        f"{X500ATTR_OID}31": "member",
        f"{X500ATTR_OID}32": "owner",
        f"{X500ATTR_OID}33": "roleOccupant",
        f"{X500ATTR_OID}36": "userCertificate",
        f"{X500ATTR_OID}37": "cACertificate",
        f"{X500ATTR_OID}38": "authorityRevocationList",
        f"{X500ATTR_OID}39": "certificateRevocationList",
        f"{X500ATTR_OID}40": "crossCertificatePair",
        f"{X500ATTR_OID}42": "givenName",
        f"{X500ATTR_OID}43": "initials",
        f"{X500ATTR_OID}44": "generationQualifier",
        f"{X500ATTR_OID}45": "x500UniqueIdentifier",
        f"{X500ATTR_OID}46": "dnQualifier",
        f"{X500ATTR_OID}47": "enhancedSearchGuide",
        f"{X500ATTR_OID}48": "protocolInformation",
        f"{X500ATTR_OID}50": "uniqueMember",
        f"{X500ATTR_OID}51": "houseIdentifier",
        f"{X500ATTR_OID}52": "supportedAlgorithms",
        f"{X500ATTR_OID}53": "deltaRevocationList",
        f"{X500ATTR_OID}54": "dmdName",
        f"{X500ATTR_OID}65": "pseudonym",
        f"{SWISSEDUPERSON_OID}1": "swissEduPersonUniqueID",
        f"{SWISSEDUPERSON_OID}2": "swissEduPersonDateOfBirth",
        f"{SWISSEDUPERSON_OID}3": "swissEduPersonGender",
        f"{SWISSEDUPERSON_OID}4": "swissEduPersonHomeOrganization",
        f"{SWISSEDUPERSON_OID}5": "swissEduPersonHomeOrganizationType",
        f"{SWISSEDUPERSON_OID}6": "swissEduPersonStudyBranch1",
        f"{SWISSEDUPERSON_OID}7": "swissEduPersonStudyBranch2",
        f"{SWISSEDUPERSON_OID}8": "swissEduPersonStudyBranch3",
        f"{SWISSEDUPERSON_OID}9": "swissEduPersonStudyLevel",
        f"{SWISSEDUPERSON_OID}10": "swissEduPersonStaffCategory",
        f"{SWISSEDUPERSON_OID}11": "swissEduPersonMatriculationNumber",
        f"{SWISSEDUPERSON_OID}12": "swissEduPersonCardUID",
        f"{SWISSEDUPERSON_OID}13": "swissEduID",
        f"{SWISSEDUPERSON_OID}1023": "swissLibraryPersonAffiliation",
        f"{SWISSEDUPERSON_OID}1025": "swissLibraryPersonResidence",
        f"{VOPERSON_V2_OID}1": "voPersonApplicationUID",
        f"{VOPERSON_V2_OID}2": "voPersonAuthorName",
        f"{VOPERSON_V2_OID}3": "voPersonCertificateDN",
        f"{VOPERSON_V2_OID}4": "voPersonCertificateIssuerDN",
        f"{VOPERSON_V2_OID}5": "voPersonExternalID",
        f"{VOPERSON_V2_OID}6": "voPersonID",
        f"{VOPERSON_V2_OID}7": "voPersonPolicyAgreement",
        f"{VOPERSON_V2_OID}8": "voPersonSoRID",
        f"{VOPERSON_V2_OID}9": "voPersonStatus",
        f"{VOPERSON_V2_OID}10": "voPersonAffiliation",
        f"{VOPERSON_V2_OID}11": "voPersonExternalAffiliation",
        f"{VOPERSON_V2_OID}12": "voPersonScopedAffiliation",
        f"{VOPERSON_V2_OID}13": "voPersonApplicationPassword",
        f"{VOPERSON_V2_OID}14": "voPersonVerifiedEmail",
        f"{VOPERSON_V2_OID}15": "voPersonToken",
    },
    "to": {
        "LegalPersonIdentifier": f"{EIDAS_LEGALPERSON}LegalPersonIdentifier",
        "LegalAddress": f"{EIDAS_LEGALPERSON}LegalPersonAddress",
        "LegalName": f"{EIDAS_LEGALPERSON}LegalName",
        "VATRegistration": f"{EIDAS_LEGALPERSON}VATRegistrationNumber",
        "TaxReference": f"{EIDAS_LEGALPERSON}TaxReference",
        "BusinessCodes": f"{EIDAS_LEGALPERSON}BusinessCodes",
        "LEI": f"{EIDAS_LEGALPERSON}LEI",
        "EORI": f"{EIDAS_LEGALPERSON}EORI",
        "SEED": f"{EIDAS_LEGALPERSON}SEED",
        "SIC": f"{EIDAS_LEGALPERSON}SIC",
        "D-2012-17-EUIdentifier": f"{EIDAS_LEGALPERSON}D-2012-17-EUIdentifier",
        "PersonIdentifier": f"{EIDAS_NATURALPERSON}PersonIdentifier",
        "FamilyName": f"{EIDAS_NATURALPERSON}CurrentFamilyName",
        "FirstName": f"{EIDAS_NATURALPERSON}CurrentGivenName",
        "DateOfBirth": f"{EIDAS_NATURALPERSON}DateOfBirth",
        "BirthName": f"{EIDAS_NATURALPERSON}BirthName",
        "PlaceOfBirth": f"{EIDAS_NATURALPERSON}PlaceOfBirth",
        "CurrentAddress": f"{EIDAS_NATURALPERSON}CurrentAddress",
        "Gender": f"{EIDAS_NATURALPERSON}Gender",
        "associatedDomain": f"{UCL_DIR_PILOT}37",
        "authContextParams": f"{DIGG_OID}3",
        "authorityRevocationList": f"{X500ATTR_OID}38",
        "authServerSignature": f"{DIGG_OID}13",
        "birthName": f"{DIGG_OID}8",
        "businessCategory": f"{X500ATTR_OID}15",
        "c": f"{X500ATTR_OID}6",
        "cACertificate": f"{X500ATTR_OID}37",
        "carLicense": f"{NETSCAPE_LDAP}1",
        "certificateRevocationList": f"{X500ATTR_OID}39",
        "cn": f"{X500ATTR_OID}3",
        "co": f"{UCL_DIR_PILOT}43",
        "countryOfCitizenship": f"{PKIX_OID}4",
        "countryOfResidence": f"{PKIX_OID}5",
        "crossCertificatePair": f"{X500ATTR_OID}40",
        "dateOfBirth": f"{PKIX_OID}1",
        "dc": f"{UCL_DIR_PILOT}25",
        "deltaRevocationList": f"{X500ATTR_OID}53",
        "departmentNumber": f"{NETSCAPE_LDAP}2",
        "destinationIndicator": f"{X500ATTR_OID}27",
        "displayName": f"{NETSCAPE_LDAP}241",
        "dmdName": f"{X500ATTR_OID}54",
        "dnQualifier": f"{X500ATTR_OID}46",
        "EAAHash": f"{UMBRELLA_EAAUser_ID}1",
        "EAAKey": f"{UMBRELLA_EAAUser_ID}3",
        "eduCourseMember": f"{EDUCOURSE_OID}2",
        "eduCourseOffering": f"{EDUCOURSE_OID}1",
        "eduPersonAffiliation": f"{EDUPERSON_OID}1",
        "eduPersonEntitlement": f"{EDUPERSON_OID}7",
        "eduPersonNickname": f"{EDUPERSON_OID}2",
        "eduPersonOrgDN": f"{EDUPERSON_OID}3",
        "eduPersonOrgUnitDN": f"{EDUPERSON_OID}4",
        "eduPersonPrimaryAffiliation": f"{EDUPERSON_OID}5",
        "eduPersonPrimaryOrgUnitDN": f"{EDUPERSON_OID}8",
        "eduPersonPrincipalName": f"{EDUPERSON_OID}6",
        "eduPersonPrincipalNamePrior": f"{EDUPERSON_OID}12",
        "eduPersonScopedAffiliation": f"{EDUPERSON_OID}9",
        "eduPersonTargetedID": f"{EDUPERSON_OID}10",
        "eduPersonAssurance": f"{EDUPERSON_OID}11",
        "eduPersonUniqueId": f"{EDUPERSON_OID}13",
        "eduPersonOrcid": f"{EDUPERSON_OID}16",
        "eidasNaturalPersonAddress": f"{DIGG_OID}9",
        "eidasPersonIdentifier": f"{DIGG_OID}7",
        "email": f"{PKCS_9}1",
        "employeeHsaId": f"{INERA_OID}1",
        "employeeNumber": f"{NETSCAPE_LDAP}3",
        "employeeType": f"{NETSCAPE_LDAP}4",
        "enhancedSearchGuide": f"{X500ATTR_OID}47",
        "facsimileTelephoneNumber": f"{X500ATTR_OID}23",
        "federationFeideSchemaVersion": f"{NOREDUPERSON_OID}9",
        "gender": f"{PKIX_OID}3",
        "generationQualifier": f"{X500ATTR_OID}44",
        "givenName": f"{X500ATTR_OID}42",
        "houseIdentifier": f"{X500ATTR_OID}51",
        "initials": f"{X500ATTR_OID}43",
        "internationaliSDNNumber": f"{X500ATTR_OID}25",
        "isMemberOf": f"{EDUMEMBER1_OID}1",
        "jpegPhoto": f"{UCL_DIR_PILOT}60",
        "knowledgeInformation": f"{X500ATTR_OID}2",
        "l": f"{X500ATTR_OID}7",
        "labeledURI": f"{UMICH}57",
        "mail": f"{UCL_DIR_PILOT}3",
        "mailLocalAddress": f"{NETSCAPE_LDAP}13",
        "member": f"{X500ATTR_OID}31",
        "norEduOrgAcronym": f"{NOREDUPERSON_OID}6",
        "norEduOrgNIN": f"{NOREDUPERSON_OID}12",
        "norEduOrgSchemaVersion": f"{NOREDUPERSON_OID}11",
        "norEduOrgUniqueIdentifier": f"{NOREDUPERSON_OID}7",
        "norEduOrgUniqueNumber": f"{NOREDUPERSON_OID}1",
        "norEduOrgUnitUniqueIdentifier": f"{NOREDUPERSON_OID}8",
        "norEduOrgUnitUniqueNumber": f"{NOREDUPERSON_OID}2",
        "norEduPersonBirthDate": f"{NOREDUPERSON_OID}3",
        "norEduPersonLIN": f"{NOREDUPERSON_OID}4",
        "norEduPersonLegalName": f"{NOREDUPERSON_OID}10",
        "norEduPersonNIN": f"{NOREDUPERSON_OID}5",
        "o": f"{X500ATTR_OID}10",
        "osiHomeUrl": f"{OPENOSI_OID}17",
        "osiPreferredTZ": f"{OPENOSI_OID}19",
        "osiICardTimeLastUpdated": f"{OPENOSI_OID}72",
        "osiMiddleName": f"{OPENOSI_OID}104",
        "osiOtherEmail": f"{OPENOSI_OID}107",
        "osiOtherHomePhone": f"{OPENOSI_OID}109",
        "osiWorkURL": f"{OPENOSI_OID}120",
        "ou": f"{X500ATTR_OID}11",
        "orgAffiliation": f"{DIGG_OID}1",
        "owner": f"{X500ATTR_OID}32",
        "pairwise-id": f"{SAML_SUBJECT_ID}pairwise-id",
        "personalIdentityNumber": f"{INERA_OID}13",
        "personalIdentityNumberBinding": f"{DIGG_OID}6",
        "physicalDeliveryOfficeName": f"{X500ATTR_OID}19",
        "placeOfBirth": f"{PKIX_OID}2",
        "postOfficeBox": f"{X500ATTR_OID}18",
        "postalAddress": f"{X500ATTR_OID}16",
        "postalCode": f"{X500ATTR_OID}17",
        "preferredDeliveryMethod": f"{X500ATTR_OID}28",
        "preferredLanguage": f"{NETSCAPE_LDAP}39",
        "presentationAddress": f"{X500ATTR_OID}29",
        "prid": f"{DIGG_OID}4",
        "pridPersistence": f"{DIGG_OID}5",
        "protocolInformation": f"{X500ATTR_OID}48",
        "pseudonym": f"{X500ATTR_OID}65",
        "PVP-USERID": f"{LDAPGVAT_UCL_DIR_PILOT}1",
        "PVP-MAIL": f"{LDAPGVAT_UCL_DIR_PILOT}3",
        "PVP-GID": f"{LDAPGVAT_OID}1",
        "PVP-BPK": f"{LDAPGVAT_OID}149",
        "PVP-OU-OKZ": f"{LDAPGVAT_OID}153",
        "PVP-VERSION": f"{LDAPGVAT_OID}261.10",
        "PVP-PRINCIPAL-NAME": f"{LDAPGVAT_OID}261.20",
        "PVP-PARTICIPANT-OKZ": f"{LDAPGVAT_OID}261.24",
        "PVP-ROLES": f"{LDAPGVAT_OID}261.30",
        "PVP-INVOICE-RECPT-ID": f"{LDAPGVAT_OID}261.40",
        "PVP-COST-CENTER-ID": f"{LDAPGVAT_OID}261.50",
        "PVP-CHARGE-CODE": f"{LDAPGVAT_OID}261.60",
        "PVP-OU-GV-OU-ID": f"{LDAPGVAT_OID}3",
        "PVP-FUNCTION": f"{LDAPGVAT_OID}33",
        "PVP-BIRTHDATE": f"{LDAPGVAT_OID}55",
        "PVP-PARTICIPANT-ID": f"{LDAPGVAT_OID}71",
        "PVP-OU": f"{LDAPGVAT_X500ATTR_OID}11",
        "PVP-TEL": f"{LDAPGVAT_X500ATTR_OID}20",
        "PVP-GIVENNAME": f"{LDAPGVAT_X500ATTR_OID}42",
        "registeredAddress": f"{X500ATTR_OID}26",
        "roleOccupant": f"{X500ATTR_OID}33",
        "sad": f"{DIGG_OID}12",
        "schacCountryOfCitizenship": f"{SCHAC}5",
        "schacCountryOfResidence": f"{SCHAC}11",
        "schacDateOfBirth": f"{SCHAC}3",
        "schacExpiryDate": f"{SCHAC}17",
        "schacGender": f"{SCHAC}2",
        "schacHomeOrganization": f"{SCHAC}9",
        "schacHomeOrganizationType": f"{SCHAC}10",
        "schacMotherTongue": f"{SCHAC}1",
        "schacPersonalPosition": f"{SCHAC}13",
        "schacPersonalTitle": f"{SCHAC}8",
        "schacPersonalUniqueCode": f"{SCHAC}14",
        "schacPersonalUniqueID": f"{SCHAC}15",
        "schacPlaceOfBirth": f"{SCHAC}4",
        "schacProjectMembership": f"{SCHAC}20",
        "schacProjectSpecificRole": f"{SCHAC}21",
        "schacSn1": f"{SCHAC}6",
        "schacSn2": f"{SCHAC}7",
        "schacUserPresenceID": f"{SCHAC}12",
        "schacUserPrivateAttribute": f"{SCHAC}18",
        "schacUserStatus": f"{SCHAC}19",
        "searchGuide": f"{X500ATTR_OID}14",
        "serialNumber": f"{X500ATTR_OID}5",
        "signMessageDigest": f"{DIGG_OID}14",
        "sisLegalGuardianFor": f"{SIS}1",
        "sisSchoolGrade": f"{SIS}2",
        "sn": f"{X500ATTR_OID}4",
        "st": f"{X500ATTR_OID}8",
        "street": f"{X500ATTR_OID}9",
        "subject-id": f"{SAML_SUBJECT_ID}subject-id",
        "supportedAlgorithms": f"{X500ATTR_OID}52",
        "supportedApplicationContext": f"{X500ATTR_OID}30",
        "telephoneNumber": f"{X500ATTR_OID}20",
        "teletexTerminalIdentifier": f"{X500ATTR_OID}22",
        "telexNumber": f"{X500ATTR_OID}21",
        "title": f"{X500ATTR_OID}12",
        "transactionIdentifier": f"{DIGG_OID}2",
        "uid": f"{UCL_DIR_PILOT}1",
        "uniqueMember": f"{X500ATTR_OID}50",
        "userCertificate": f"{X500ATTR_OID}36",
        #  'userCertificate': DIGG_OID+'10',
        "userPKCS12": f"{NETSCAPE_LDAP}216",
        "userSMIMECertificate": f"{NETSCAPE_LDAP}40",
        "userSignature": f"{DIGG_OID}11",
        "x121Address": f"{X500ATTR_OID}24",
        "x500UniqueIdentifier": f"{X500ATTR_OID}45",
        "swissEduPersonUniqueID": f"{SWISSEDUPERSON_OID}1",
        "swissEduPersonDateOfBirth": f"{SWISSEDUPERSON_OID}2",
        "swissEduPersonGender": f"{SWISSEDUPERSON_OID}3",
        "swissEduPersonHomeOrganization": f"{SWISSEDUPERSON_OID}4",
        "swissEduPersonHomeOrganizationType": f"{SWISSEDUPERSON_OID}5",
        "swissEduPersonStudyBranch1": f"{SWISSEDUPERSON_OID}6",
        "swissEduPersonStudyBranch2": f"{SWISSEDUPERSON_OID}7",
        "swissEduPersonStudyBranch3": f"{SWISSEDUPERSON_OID}8",
        "swissEduPersonStudyLevel": f"{SWISSEDUPERSON_OID}9",
        "swissEduPersonStaffCategory": f"{SWISSEDUPERSON_OID}10",
        "swissEduPersonMatriculationNumber": f"{SWISSEDUPERSON_OID}11",
        "swissEduPersonCardUID": f"{SWISSEDUPERSON_OID}12",
        "swissEduID": f"{SWISSEDUPERSON_OID}13",
        "swissLibraryPersonAffiliation": f"{SWISSEDUPERSON_OID}1023",
        "swissLibraryPersonResidence": f"{SWISSEDUPERSON_OID}1025",
        "voPersonAffiliation": f"{VOPERSON_V2_OID}10",
        "voPersonApplicationPassword": f"{VOPERSON_V2_OID}13",
        "voPersonApplicationUID": f"{VOPERSON_V2_OID}1",
        "voPersonAuthorName": f"{VOPERSON_V2_OID}2",
        "voPersonCertificateDN": f"{VOPERSON_V2_OID}3",
        "voPersonCertificateIssuerDN": f"{VOPERSON_V2_OID}4",
        "voPersonExternalAffiliation": f"{VOPERSON_V2_OID}11",
        "voPersonExternalID": f"{VOPERSON_V2_OID}5",
        "voPersonID": f"{VOPERSON_V2_OID}6",
        "voPersonPolicyAgreement": f"{VOPERSON_V2_OID}7",
        "voPersonScopedAffiliation": f"{VOPERSON_V2_OID}12",
        "voPersonSoRID": f"{VOPERSON_V2_OID}8",
        "voPersonStatus": f"{VOPERSON_V2_OID}9",
        "voPersonToken": f"{VOPERSON_V2_OID}15",
        "voPersonVerifiedEmail": f"{VOPERSON_V2_OID}14",
    },
}



#!/usr/bin/env python

#
# Generated Mon May  2 14:23:33 2011 by parse_xsd.py version 0.4.
#
# A summary of available specifications can be found at:
# https://wiki.oasis-open.org/security/FrontPage
#
# saml core specifications to be found at:
# if any question arise please query the following pdf.
# http://docs.oasis-open.org/security/saml/v2.0/saml-core-2.0-os.pdf
# The specification was later updated with errata, and the new version is here:
# https://www.oasis-open.org/committees/download.php/56776/sstc-saml-core-errata-2.0-wd-07.pdf
#


import base64
from datetime import date
from datetime import datetime

import saml2
from saml2 import SamlBase
from saml2 import xmldsig as ds
from saml2 import xmlenc as xenc
from saml2.validate import MustValueError
from saml2.validate import ShouldValueError
from saml2.validate import valid_domain_name
from saml2.validate import valid_ipv4
from saml2.validate import valid_ipv6


# authentication information fields
NAMESPACE = "urn:oasis:names:tc:SAML:2.0:assertion"

# xmlschema definition
XSD = "xs"
# xmlschema templates and extensions
XS_NAMESPACE = "http://www.w3.org/2001/XMLSchema"
# xmlschema-instance, which contains several builtin attributes
XSI_NAMESPACE = "http://www.w3.org/2001/XMLSchema-instance"
# xml soap namespace
NS_SOAP_ENC = "http://schemas.xmlsoap.org/soap/encoding/"
# type definitions for xmlschemas
XSI_TYPE = "{%s}type" % XSI_NAMESPACE
# nil type definition for xmlschemas
XSI_NIL = "{%s}nil" % XSI_NAMESPACE

# idp and sp communicate usually about a subject(NameID)
# the format determines the category the subject is in

# custom subject
NAMEID_FORMAT_UNSPECIFIED = "urn:oasis:names:tc:SAML:1.1:nameid-format:unspecified"
# subject as email address
NAMEID_FORMAT_EMAILADDRESS = "urn:oasis:names:tc:SAML:1.1:nameid-format:emailAddress"
# subject as x509 key
NAMEID_FORMAT_X509SUBJECTNAME = "urn:oasis:names:tc:SAML:1.1:nameid-format:X509SubjectName"
# subject as windows domain name
NAMEID_FORMAT_WINDOWSDOMAINQUALIFIEDNAME = "urn:oasis:names:tc:SAML:1.1:nameid-format:WindowsDomainQualifiedName"
# subject from a kerberos instance
NAMEID_FORMAT_KERBEROS = "urn:oasis:names:tc:SAML:2.0:nameid-format:kerberos"
# subject as name
NAMEID_FORMAT_ENTITY = "urn:oasis:names:tc:SAML:2.0:nameid-format:entity"
# linked subject
NAMEID_FORMAT_PERSISTENT = "urn:oasis:names:tc:SAML:2.0:nameid-format:persistent"
# annonymous subject
NAMEID_FORMAT_TRANSIENT = "urn:oasis:names:tc:SAML:2.0:nameid-format:transient"
# subject avaiable in encrypted format
NAMEID_FORMAT_ENCRYPTED = "urn:oasis:names:tc:SAML:2.0:nameid-format:encrypted"
# dicc for avaiable formats
NAMEID_FORMATS_SAML2 = (
    ("NAMEID_FORMAT_EMAILADDRESS", NAMEID_FORMAT_EMAILADDRESS),
    ("NAMEID_FORMAT_ENCRYPTED", NAMEID_FORMAT_ENCRYPTED),
    ("NAMEID_FORMAT_ENTITY", NAMEID_FORMAT_ENTITY),
    ("NAMEID_FORMAT_PERSISTENT", NAMEID_FORMAT_PERSISTENT),
    ("NAMEID_FORMAT_TRANSIENT", NAMEID_FORMAT_TRANSIENT),
    ("NAMEID_FORMAT_UNSPECIFIED", NAMEID_FORMAT_UNSPECIFIED),
)

# a profile outlines a set of rules describing how to embed SAML assertions.
# https://docs.oasis-open.org/security/saml/v2.0/saml-profiles-2.0-os.pdf
# The specification was later updated with errata, and the new version is here:
# https://www.oasis-open.org/committees/download.php/56782/sstc-saml-profiles-errata-2.0-wd-07.pdf

# XML based values for SAML attributes
PROFILE_ATTRIBUTE_BASIC = "urn:oasis:names:tc:SAML:2.0:profiles:attribute:basic"

# an AuthnRequest is made to initiate authentication
# authenticate the request with login credentials
AUTHN_PASSWORD = "urn:oasis:names:tc:SAML:2.0:ac:classes:Password"
# authenticate the request with login credentials, over tls/https
AUTHN_PASSWORD_PROTECTED = "urn:oasis:names:tc:SAML:2.0:ac:classes:PasswordProtectedTransport"

# attribute statements is key:value metadata shared with your app

# custom format
NAME_FORMAT_UNSPECIFIED = "urn:oasis:names:tc:SAML:2.0:attrname-format:unspecified"
# uri format
NAME_FORMAT_URI = "urn:oasis:names:tc:SAML:2.0:attrname-format:uri"
# XML-based format
NAME_FORMAT_BASIC = "urn:oasis:names:tc:SAML:2.0:attrname-format:basic"
# dicc for avaiable formats
NAME_FORMATS_SAML2 = (
    ("NAME_FORMAT_BASIC", NAME_FORMAT_BASIC),
    ("NAME_FORMAT_URI", NAME_FORMAT_URI),
    ("NAME_FORMAT_UNSPECIFIED", NAME_FORMAT_UNSPECIFIED),
)

# the SAML authority's decision can be predetermined by arbitrary context

# the specified action is permitted
DECISION_TYPE_PERMIT = "Permit"
# the specified action is denied
DECISION_TYPE_DENY = "Deny"
# the SAML authority cannot determine if the action is permitted or denied
DECISION_TYPE_INDETERMINATE = "Indeterminate"


# consent attributes determine wether consent has been given and under
# what conditions

# no claim to consent is made
CONSENT_UNSPECIFIED = "urn:oasis:names:tc:SAML:2.0:consent:unspecified"
# consent has been obtained
CONSENT_OBTAINED = "urn:oasis:names:tc:SAML:2.0:consent:obtained"
# consent has been obtained before the message has been initiated
CONSENT_PRIOR = "urn:oasis:names:tc:SAML:2.0:consent:prior"
# consent has been obtained implicitly
CONSENT_IMPLICIT = "urn:oasis:names:tc:SAML:2.0:consent:current-implicit"
# consent has been obtained explicitly
CONSENT_EXPLICIT = "urn:oasis:names:tc:SAML:2.0:consent:current-explicit"
# no consent has been obtained
CONSENT_UNAVAILABLE = "urn:oasis:names:tc:SAML:2.0:consent:unavailable"
# no consent is needed.
CONSENT_INAPPLICABLE = "urn:oasis:names:tc:SAML:2.0:consent:inapplicable"


# Subject confirmation methods(scm), can be issued, besides the subject itself
# by third parties.
# http://docs.oasis-open.org/wss/oasis-wss-saml-token-profile-1.0.pdf

# the 3rd party is identified on behalf of the subject given private/public key
SCM_HOLDER_OF_KEY = "urn:oasis:names:tc:SAML:2.0:cm:holder-of-key"
# the 3rd party is identified by subject confirmation and must include a security header
# signing its content.
SCM_SENDER_VOUCHES = "urn:oasis:names:tc:SAML:2.0:cm:sender-vouches"
# a bearer token is issued instead.
SCM_BEARER = "urn:oasis:names:tc:SAML:2.0:cm:bearer"


class AttributeValueBase(SamlBase):
    def __init__(self, text=None, extension_elements=None, extension_attributes=None):
        self._extatt = {}

        SamlBase.__init__(
            self, text=None, extension_elements=extension_elements, extension_attributes=extension_attributes
        )
        if self._extatt:
            self.extension_attributes = self._extatt

        if text:
            self.set_text(text)
        elif not extension_elements:
            self.extension_attributes = {XSI_NIL: "true"}
        elif XSI_TYPE in self.extension_attributes:
            del self.extension_attributes[XSI_TYPE]

    def __setattr__(self, key, value):
        if key == "text":
            self.set_text(value)
        else:
            SamlBase.__setattr__(self, key, value)

    def verify(self):
        if not self.text and not self.extension_elements:
            if not self.extension_attributes:
                raise Exception("Attribute value base should not have extension attributes")
            if self.extension_attributes[XSI_NIL] != "true":
                raise Exception("Attribute value base should not have extension attributes")
            return True
        else:
            SamlBase.verify(self)

    def set_type(self, typ):
        try:
            del self.extension_attributes[XSI_NIL]
        except (AttributeError, KeyError):
            pass

        try:
            self.extension_attributes[XSI_TYPE] = typ
        except AttributeError:
            self._extatt[XSI_TYPE] = typ

        if typ.startswith("xs:"):
            try:
                self.extension_attributes["xmlns:xs"] = XS_NAMESPACE
            except AttributeError:
                self._extatt["xmlns:xs"] = XS_NAMESPACE

        if typ.startswith("xsd:"):
            try:
                self.extension_attributes["xmlns:xsd"] = XS_NAMESPACE
            except AttributeError:
                self._extatt["xmlns:xsd"] = XS_NAMESPACE

    def get_type(self):
        try:
            return self.extension_attributes[XSI_TYPE]
        except (KeyError, AttributeError):
            try:
                return self._extatt[XSI_TYPE]
            except KeyError:
                return ""

    def clear_type(self):
        try:
            del self.extension_attributes[XSI_TYPE]
        except KeyError:
            pass
        try:
            del self._extatt[XSI_TYPE]
        except KeyError:
            pass

    def set_text(self, value, base64encode=False):
        def _wrong_type_value(xsd, value):
            msg = "Type and value do not match: {xsd}:{type}:{value}"
            msg = msg.format(xsd=xsd, type=type(value), value=value)
            raise ValueError(msg)

        if isinstance(value, bytes):
            value = value.decode("utf-8")

        type_to_xsd = {
            str: "string",
            int: "integer",
            float: "float",
            bool: "boolean",
            type(None): "",
        }

        # entries of xsd-types each declaring:
        # - a corresponding python type
        # - a function to turn a string into that type
        # - a function to turn that type into a text-value
        xsd_types_props = {
            "string": {
                "type": str,
                "to_type": str,
                "to_text": str,
            },
            "integer": {
                "type": int,
                "to_type": int,
                "to_text": str,
            },
            "short": {
                "type": int,
                "to_type": int,
                "to_text": str,
            },
            "int": {
                "type": int,
                "to_type": int,
                "to_text": str,
            },
            "long": {
                "type": int,
                "to_type": int,
                "to_text": str,
            },
            "float": {
                "type": float,
                "to_type": float,
                "to_text": str,
            },
            "double": {
                "type": float,
                "to_type": float,
                "to_text": str,
            },
            "boolean": {
                "type": bool,
                "to_type": lambda x: {
                    "true": True,
                    "false": False,
                }[str(x).lower()],
                "to_text": lambda x: str(x).lower(),
            },
            "date": {
                "type": date,
                "to_type": lambda x: datetime.strptime(x, "%Y-%m-%d").date(),
                "to_text": str,
            },
            "base64Binary": {
                "type": str,
                "to_type": str,
                "to_text": (lambda x: base64.encodebytes(x.encode()) if base64encode else x),
            },
            "anyType": {
                "type": type(value),
                "to_type": lambda x: x,
                "to_text": lambda x: x,
            },
            "": {
                "type": type(None),
                "to_type": lambda x: None,
                "to_text": lambda x: "",
            },
        }

        xsd_string = "base64Binary" if base64encode else self.get_type() or type_to_xsd.get(type(value))

        xsd_ns, xsd_type = (
            ["", type(None)]
            if xsd_string is None
            else ["", ""]
            if xsd_string == ""
            else [XSD if xsd_string in xsd_types_props else "", xsd_string]
            if ":" not in xsd_string
            else xsd_string.split(":", 1)
        )

        xsd_type_props = xsd_types_props.get(xsd_type)
        if not xsd_type_props:
            xsd_type_props = xsd_types_props.get("string")

        valid_type = xsd_type_props.get("type", type(None))
        to_type = xsd_type_props.get("to_type", str)
        to_text = xsd_type_props.get("to_text", str)

        # cast to correct type before type-checking
        if type(value) is str and valid_type is not str:
            try:
                value = to_type(value)
            except (TypeError, ValueError, KeyError):
                # the cast failed
                _wrong_type_value(xsd=xsd_type, value=value)

        if type(value) is not valid_type:
            _wrong_type_value(xsd=xsd_type, value=value)

        text = to_text(value)
        self.set_type(f"{xsd_ns}:{xsd_type}" if xsd_ns else xsd_type if xsd_type else "")
        SamlBase.__setattr__(self, "text", text)
        return self

    def harvest_element_tree(self, tree):
        # Fill in the instance members from the contents of the XML tree.
        for child in tree:
            self._convert_element_tree_to_member(child)
        for attribute, value in iter(tree.attrib.items()):
            self._convert_element_attribute_to_member(attribute, value)

        # if we have added children to this node
        # we consider whitespace insignificant
        # and remove/trim/strip whitespace
        # and expect to not have actual text content
        text = tree.text.strip() if tree.text and self.extension_elements else tree.text
        if text:
            # print("set_text:", tree.text)
            # clear type
            # self.clear_type()
            self.set_text(text)

        # if we have added a text node
        # or other children to this node
        # remove the nil marker
        if text or self.extension_elements:
            if XSI_NIL in self.extension_attributes:
                del self.extension_attributes[XSI_NIL]


class BaseIDAbstractType_(SamlBase):
    """The urn:oasis:names:tc:SAML:2.0:assertion:BaseIDAbstractType element"""

    c_tag = "BaseIDAbstractType"
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_attributes["NameQualifier"] = ("name_qualifier", "string", False)
    c_attributes["SPNameQualifier"] = ("sp_name_qualifier", "string", False)

    def __init__(
        self, name_qualifier=None, sp_name_qualifier=None, text=None, extension_elements=None, extension_attributes=None
    ):
        SamlBase.__init__(
            self, text=text, extension_elements=extension_elements, extension_attributes=extension_attributes
        )
        self.name_qualifier = name_qualifier
        self.sp_name_qualifier = sp_name_qualifier


class NameIDType_(SamlBase):
    """The urn:oasis:names:tc:SAML:2.0:assertion:NameIDType element"""

    c_tag = "NameIDType"
    c_namespace = NAMESPACE
    c_value_type = {"base": "string"}
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_attributes["NameQualifier"] = ("name_qualifier", "string", False)
    c_attributes["SPNameQualifier"] = ("sp_name_qualifier", "string", False)
    c_attributes["Format"] = ("format", "anyURI", False)
    c_attributes["SPProvidedID"] = ("sp_provided_id", "string", False)

    def __init__(
        self,
        name_qualifier=None,
        sp_name_qualifier=None,
        format=None,
        sp_provided_id=None,
        text=None,
        extension_elements=None,
        extension_attributes=None,
    ):
        SamlBase.__init__(
            self, text=text, extension_elements=extension_elements, extension_attributes=extension_attributes
        )
        self.name_qualifier = name_qualifier
        self.sp_name_qualifier = sp_name_qualifier
        self.format = format
        self.sp_provided_id = sp_provided_id


def name_id_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(NameIDType_, xml_string)


class EncryptedElementType_(SamlBase):
    """The urn:oasis:names:tc:SAML:2.0:assertion:EncryptedElementType element"""

    c_tag = "EncryptedElementType"
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_children["{http://www.w3.org/2001/04/xmlenc#}EncryptedData"] = ("encrypted_data", xenc.EncryptedData)
    c_children["{http://www.w3.org/2001/04/xmlenc#}EncryptedKey"] = ("encrypted_key", [xenc.EncryptedKey])
    c_cardinality["encrypted_key"] = {"min": 0}
    c_child_order.extend(["encrypted_data", "encrypted_key"])

    def __init__(
        self, encrypted_data=None, encrypted_key=None, text=None, extension_elements=None, extension_attributes=None
    ):
        SamlBase.__init__(
            self, text=text, extension_elements=extension_elements, extension_attributes=extension_attributes
        )
        self.encrypted_data = encrypted_data
        self.encrypted_key = encrypted_key or []


def encrypted_element_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(EncryptedElementType_, xml_string)


class EncryptedID(EncryptedElementType_):
    """The urn:oasis:names:tc:SAML:2.0:assertion:EncryptedID element"""

    c_tag = "EncryptedID"
    c_namespace = NAMESPACE
    c_children = EncryptedElementType_.c_children.copy()
    c_attributes = EncryptedElementType_.c_attributes.copy()
    c_child_order = EncryptedElementType_.c_child_order[:]
    c_cardinality = EncryptedElementType_.c_cardinality.copy()


def encrypted_id_from_string(xml_string):
    return saml2.create_class_from_xml_string(EncryptedID, xml_string)


class Issuer(NameIDType_):
    """The urn:oasis:names:tc:SAML:2.0:assertion:Issuer element"""

    c_tag = "Issuer"
    c_namespace = NAMESPACE
    c_children = NameIDType_.c_children.copy()
    c_attributes = NameIDType_.c_attributes.copy()
    c_child_order = NameIDType_.c_child_order[:]
    c_cardinality = NameIDType_.c_cardinality.copy()


def issuer_from_string(xml_string):
    return saml2.create_class_from_xml_string(Issuer, xml_string)


class AssertionIDRef(SamlBase):
    """The urn:oasis:names:tc:SAML:2.0:assertion:AssertionIDRef element"""

    c_tag = "AssertionIDRef"
    c_namespace = NAMESPACE
    c_value_type = {"base": "NCName"}
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()


def assertion_id_ref_from_string(xml_string):
    return saml2.create_class_from_xml_string(AssertionIDRef, xml_string)


class AssertionURIRef(SamlBase):
    """The urn:oasis:names:tc:SAML:2.0:assertion:AssertionURIRef element"""

    c_tag = "AssertionURIRef"
    c_namespace = NAMESPACE
    c_value_type = {"base": "anyURI"}
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()


def assertion_uri_ref_from_string(xml_string):
    return saml2.create_class_from_xml_string(AssertionURIRef, xml_string)


class SubjectConfirmationDataType_(SamlBase):
    """The urn:oasis:names:tc:SAML:2.0:assertion:SubjectConfirmationDataType
    element"""

    c_tag = "SubjectConfirmationDataType"
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_attributes["NotBefore"] = ("not_before", "dateTime", False)
    c_attributes["NotOnOrAfter"] = ("not_on_or_after", "dateTime", False)
    c_attributes["Recipient"] = ("recipient", "anyURI", False)
    c_attributes["InResponseTo"] = ("in_response_to", "NCName", False)
    c_attributes["Address"] = ("address", "string", False)
    c_any = {"namespace": "##any", "processContents": "lax", "minOccurs": "0", "maxOccurs": "unbounded"}
    c_any_attribute = {"namespace": "##other", "processContents": "lax"}

    def __init__(
        self,
        not_before=None,
        not_on_or_after=None,
        recipient=None,
        in_response_to=None,
        address=None,
        text=None,
        extension_elements=None,
        extension_attributes=None,
    ):
        SamlBase.__init__(
            self, text=text, extension_elements=extension_elements, extension_attributes=extension_attributes
        )
        self.not_before = not_before
        self.not_on_or_after = not_on_or_after
        self.recipient = recipient
        self.in_response_to = in_response_to
        self.address = address


def subject_confirmation_data_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(SubjectConfirmationDataType_, xml_string)


class KeyInfoConfirmationDataType_(SamlBase):
    """The urn:oasis:names:tc:SAML:2.0:assertion:KeyInfoConfirmationDataType
    element"""

    c_tag = "KeyInfoConfirmationDataType"
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_children["{http://www.w3.org/2000/09/xmldsig#}KeyInfo"] = ("key_info", [ds.KeyInfo])
    c_cardinality["key_info"] = {"min": 1}
    c_child_order.extend(["key_info"])

    def __init__(self, key_info=None, text=None, extension_elements=None, extension_attributes=None):
        SamlBase.__init__(
            self, text=text, extension_elements=extension_elements, extension_attributes=extension_attributes
        )
        self.key_info = key_info or []


def key_info_confirmation_data_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(KeyInfoConfirmationDataType_, xml_string)


class ConditionAbstractType_(SamlBase):
    """The urn:oasis:names:tc:SAML:2.0:assertion:ConditionAbstractType
    element"""

    c_tag = "ConditionAbstractType"
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()


class Audience(SamlBase):
    """The urn:oasis:names:tc:SAML:2.0:assertion:Audience element"""

    c_tag = "Audience"
    c_namespace = NAMESPACE
    c_value_type = {"base": "anyURI"}
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()


def audience_from_string(xml_string):
    return saml2.create_class_from_xml_string(Audience, xml_string)


class OneTimeUseType_(ConditionAbstractType_):
    """The urn:oasis:names:tc:SAML:2.0:assertion:OneTimeUseType element"""

    c_tag = "OneTimeUseType"
    c_namespace = NAMESPACE
    c_children = ConditionAbstractType_.c_children.copy()
    c_attributes = ConditionAbstractType_.c_attributes.copy()
    c_child_order = ConditionAbstractType_.c_child_order[:]
    c_cardinality = ConditionAbstractType_.c_cardinality.copy()


def one_time_use_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(OneTimeUseType_, xml_string)


class ProxyRestrictionType_(ConditionAbstractType_):
    """The urn:oasis:names:tc:SAML:2.0:assertion:ProxyRestrictionType element"""

    c_tag = "ProxyRestrictionType"
    c_namespace = NAMESPACE
    c_children = ConditionAbstractType_.c_children.copy()
    c_attributes = ConditionAbstractType_.c_attributes.copy()
    c_child_order = ConditionAbstractType_.c_child_order[:]
    c_cardinality = ConditionAbstractType_.c_cardinality.copy()
    c_children["{urn:oasis:names:tc:SAML:2.0:assertion}Audience"] = ("audience", [Audience])
    c_cardinality["audience"] = {"min": 0}
    c_attributes["Count"] = ("count", "nonNegativeInteger", False)
    c_child_order.extend(["audience"])

    def __init__(self, audience=None, count=None, text=None, extension_elements=None, extension_attributes=None):
        ConditionAbstractType_.__init__(
            self, text=text, extension_elements=extension_elements, extension_attributes=extension_attributes
        )
        self.audience = audience or []
        self.count = count


def proxy_restriction_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(ProxyRestrictionType_, xml_string)


class EncryptedAssertion(EncryptedElementType_):
    """The urn:oasis:names:tc:SAML:2.0:assertion:EncryptedAssertion element"""

    c_tag = "EncryptedAssertion"
    c_namespace = NAMESPACE
    c_children = EncryptedElementType_.c_children.copy()
    c_attributes = EncryptedElementType_.c_attributes.copy()
    c_child_order = EncryptedElementType_.c_child_order[:]
    c_cardinality = EncryptedElementType_.c_cardinality.copy()


def encrypted_assertion_from_string(xml_string):
    return saml2.create_class_from_xml_string(EncryptedAssertion, xml_string)


class StatementAbstractType_(SamlBase):
    """The urn:oasis:names:tc:SAML:2.0:assertion:StatementAbstractType element"""

    c_tag = "StatementAbstractType"
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()


class SubjectLocalityType_(SamlBase):
    """The urn:oasis:names:tc:SAML:2.0:assertion:SubjectLocalityType element"""

    c_tag = "SubjectLocalityType"
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_attributes["Address"] = ("address", "string", False)
    c_attributes["DNSName"] = ("dns_name", "string", False)

    def __init__(self, address=None, dns_name=None, text=None, extension_elements=None, extension_attributes=None):
        SamlBase.__init__(
            self, text=text, extension_elements=extension_elements, extension_attributes=extension_attributes
        )
        self.address = address
        self.dns_name = dns_name


def subject_locality_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(SubjectLocalityType_, xml_string)


class AuthnContextClassRef(SamlBase):
    """The urn:oasis:names:tc:SAML:2.0:assertion:AuthnContextClassRef element"""

    c_tag = "AuthnContextClassRef"
    c_namespace = NAMESPACE
    c_value_type = {"base": "anyURI"}
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()


def authn_context_class_ref_from_string(xml_string):
    return saml2.create_class_from_xml_string(AuthnContextClassRef, xml_string)


class AuthnContextDeclRef(SamlBase):
    """The urn:oasis:names:tc:SAML:2.0:assertion:AuthnContextDeclRef element"""

    c_tag = "AuthnContextDeclRef"
    c_namespace = NAMESPACE
    c_value_type = {"base": "anyURI"}
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()


def authn_context_decl_ref_from_string(xml_string):
    return saml2.create_class_from_xml_string(AuthnContextDeclRef, xml_string)


class AuthnContextDecl(SamlBase):
    """The urn:oasis:names:tc:SAML:2.0:assertion:AuthnContextDecl element"""

    c_tag = "AuthnContextDecl"
    c_namespace = NAMESPACE
    c_value_type = {"base": "anyType"}
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()


def authn_context_decl_from_string(xml_string):
    return saml2.create_class_from_xml_string(AuthnContextDecl, xml_string)


class AuthenticatingAuthority(SamlBase):
    """The urn:oasis:names:tc:SAML:2.0:assertion:AuthenticatingAuthority
    element"""

    c_tag = "AuthenticatingAuthority"
    c_namespace = NAMESPACE
    c_value_type = {"base": "anyURI"}
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()


def authenticating_authority_from_string(xml_string):
    return saml2.create_class_from_xml_string(AuthenticatingAuthority, xml_string)


class DecisionType_(SamlBase):
    """The urn:oasis:names:tc:SAML:2.0:assertion:DecisionType element"""

    c_tag = "DecisionType"
    c_namespace = NAMESPACE
    c_value_type = {"base": "string", "enumeration": ["Permit", "Deny", "Indeterminate"]}
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()


def decision_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(DecisionType_, xml_string)


class ActionType_(SamlBase):
    """The urn:oasis:names:tc:SAML:2.0:assertion:ActionType element"""

    c_tag = "ActionType"
    c_namespace = NAMESPACE
    c_value_type = {"base": "string"}
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_attributes["Namespace"] = ("namespace", "anyURI", True)

    def __init__(self, namespace=None, text=None, extension_elements=None, extension_attributes=None):
        SamlBase.__init__(
            self, text=text, extension_elements=extension_elements, extension_attributes=extension_attributes
        )
        self.namespace = namespace


def action_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(ActionType_, xml_string)


class AttributeValue(AttributeValueBase):
    """The urn:oasis:names:tc:SAML:2.0:assertion:AttributeValue element"""

    c_tag = "AttributeValue"
    c_namespace = NAMESPACE
    c_value_type = {"base": "anyType"}
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()


def attribute_value_from_string(xml_string):
    return saml2.create_class_from_xml_string(AttributeValue, xml_string)


class EncryptedAttribute(EncryptedElementType_):
    """The urn:oasis:names:tc:SAML:2.0:assertion:EncryptedAttribute element"""

    c_tag = "EncryptedAttribute"
    c_namespace = NAMESPACE
    c_children = EncryptedElementType_.c_children.copy()
    c_attributes = EncryptedElementType_.c_attributes.copy()
    c_child_order = EncryptedElementType_.c_child_order[:]
    c_cardinality = EncryptedElementType_.c_cardinality.copy()


def encrypted_attribute_from_string(xml_string):
    return saml2.create_class_from_xml_string(EncryptedAttribute, xml_string)


class BaseID(BaseIDAbstractType_):
    """The urn:oasis:names:tc:SAML:2.0:assertion:BaseID element"""

    c_tag = "BaseID"
    c_namespace = NAMESPACE
    c_children = BaseIDAbstractType_.c_children.copy()
    c_attributes = BaseIDAbstractType_.c_attributes.copy()
    c_child_order = BaseIDAbstractType_.c_child_order[:]
    c_cardinality = BaseIDAbstractType_.c_cardinality.copy()


def base_id_from_string(xml_string):
    return saml2.create_class_from_xml_string(BaseID, xml_string)


class NameID(NameIDType_):
    """The urn:oasis:names:tc:SAML:2.0:assertion:NameID element

    From the Oasis SAML2 Technical Overview:

    "The <NameID> element within a <Subject> offers the ability to provide name
     identifiers in a number of different formats. SAML's predefined formats
     include: Email address, X.509 subject name, Windows domain qualified name,
     Kerberos principal name, Entity identifier, Persistent identifier,
     Transient identifier."
    """

    c_tag = "NameID"
    c_namespace = NAMESPACE
    c_children = NameIDType_.c_children.copy()
    c_attributes = NameIDType_.c_attributes.copy()
    c_child_order = NameIDType_.c_child_order[:]
    c_cardinality = NameIDType_.c_cardinality.copy()


def name_id_from_string(xml_string):
    return saml2.create_class_from_xml_string(NameID, xml_string)


class SubjectConfirmationData(SubjectConfirmationDataType_):
    """The urn:oasis:names:tc:SAML:2.0:assertion:SubjectConfirmationData
    element"""

    c_tag = "SubjectConfirmationData"
    c_namespace = NAMESPACE
    c_children = SubjectConfirmationDataType_.c_children.copy()
    c_attributes = SubjectConfirmationDataType_.c_attributes.copy()
    c_child_order = SubjectConfirmationDataType_.c_child_order[:]
    c_cardinality = SubjectConfirmationDataType_.c_cardinality.copy()


def subject_confirmation_data_from_string(xml_string):
    return saml2.create_class_from_xml_string(SubjectConfirmationData, xml_string)


class Condition(ConditionAbstractType_):
    """The urn:oasis:names:tc:SAML:2.0:assertion:Condition element"""

    c_tag = "Condition"
    c_namespace = NAMESPACE
    c_children = ConditionAbstractType_.c_children.copy()
    c_attributes = ConditionAbstractType_.c_attributes.copy()
    c_child_order = ConditionAbstractType_.c_child_order[:]
    c_cardinality = ConditionAbstractType_.c_cardinality.copy()


def condition_from_string(xml_string):
    return saml2.create_class_from_xml_string(Condition, xml_string)


class AudienceRestrictionType_(ConditionAbstractType_):
    """The urn:oasis:names:tc:SAML:2.0:assertion:AudienceRestrictionType
    element"""

    c_tag = "AudienceRestrictionType"
    c_namespace = NAMESPACE
    c_children = ConditionAbstractType_.c_children.copy()
    c_attributes = ConditionAbstractType_.c_attributes.copy()
    c_child_order = ConditionAbstractType_.c_child_order[:]
    c_cardinality = ConditionAbstractType_.c_cardinality.copy()
    c_children["{urn:oasis:names:tc:SAML:2.0:assertion}Audience"] = ("audience", [Audience])
    c_cardinality["audience"] = {"min": 1}
    c_child_order.extend(["audience"])

    def __init__(self, audience=None, text=None, extension_elements=None, extension_attributes=None):
        ConditionAbstractType_.__init__(
            self, text=text, extension_elements=extension_elements, extension_attributes=extension_attributes
        )
        self.audience = audience or []


def audience_restriction_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(AudienceRestrictionType_, xml_string)


class OneTimeUse(OneTimeUseType_):
    """The urn:oasis:names:tc:SAML:2.0:assertion:OneTimeUse element"""

    c_tag = "OneTimeUse"
    c_namespace = NAMESPACE
    c_children = OneTimeUseType_.c_children.copy()
    c_attributes = OneTimeUseType_.c_attributes.copy()
    c_child_order = OneTimeUseType_.c_child_order[:]
    c_cardinality = OneTimeUseType_.c_cardinality.copy()


def one_time_use_from_string(xml_string):
    return saml2.create_class_from_xml_string(OneTimeUse, xml_string)


class ProxyRestriction(ProxyRestrictionType_):
    """The urn:oasis:names:tc:SAML:2.0:assertion:ProxyRestriction element"""

    c_tag = "ProxyRestriction"
    c_namespace = NAMESPACE
    c_children = ProxyRestrictionType_.c_children.copy()
    c_attributes = ProxyRestrictionType_.c_attributes.copy()
    c_child_order = ProxyRestrictionType_.c_child_order[:]
    c_cardinality = ProxyRestrictionType_.c_cardinality.copy()


def proxy_restriction_from_string(xml_string):
    return saml2.create_class_from_xml_string(ProxyRestriction, xml_string)


class Statement(StatementAbstractType_):
    """The urn:oasis:names:tc:SAML:2.0:assertion:Statement element"""

    c_tag = "Statement"
    c_namespace = NAMESPACE
    c_children = StatementAbstractType_.c_children.copy()
    c_attributes = StatementAbstractType_.c_attributes.copy()
    c_child_order = StatementAbstractType_.c_child_order[:]
    c_cardinality = StatementAbstractType_.c_cardinality.copy()


def statement_from_string(xml_string):
    return saml2.create_class_from_xml_string(Statement, xml_string)


class SubjectLocality(SubjectLocalityType_):
    """The urn:oasis:names:tc:SAML:2.0:assertion:SubjectLocality element"""

    c_tag = "SubjectLocality"
    c_namespace = NAMESPACE
    c_children = SubjectLocalityType_.c_children.copy()
    c_attributes = SubjectLocalityType_.c_attributes.copy()
    c_child_order = SubjectLocalityType_.c_child_order[:]
    c_cardinality = SubjectLocalityType_.c_cardinality.copy()

    def verify(self):
        if self.address:
            # dotted-decimal IPv4 or RFC3513 IPv6 address
            if valid_ipv4(self.address) or valid_ipv6(self.address):
                pass
            else:
                raise ShouldValueError("Not an IPv4 or IPv6 address")
        elif self.dns_name:
            valid_domain_name(self.dns_name)

        return SubjectLocalityType_.verify(self)


def subject_locality_from_string(xml_string):
    return saml2.create_class_from_xml_string(SubjectLocality, xml_string)


class AuthnContextType_(SamlBase):
    """The urn:oasis:names:tc:SAML:2.0:assertion:AuthnContextType element"""

    c_tag = "AuthnContextType"
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_children["{urn:oasis:names:tc:SAML:2.0:assertion}AuthnContextClassRef"] = (
        "authn_context_class_ref",
        AuthnContextClassRef,
    )
    c_children["{urn:oasis:names:tc:SAML:2.0:assertion}AuthnContextDecl"] = ("authn_context_decl", AuthnContextDecl)
    c_cardinality["authn_context_decl"] = {"min": 0, "max": 1}
    c_children["{urn:oasis:names:tc:SAML:2.0:assertion}AuthnContextDeclRef"] = (
        "authn_context_decl_ref",
        AuthnContextDeclRef,
    )
    c_cardinality["authn_context_decl_ref"] = {"min": 0, "max": 1}
    c_children["{urn:oasis:names:tc:SAML:2.0:assertion}AuthenticatingAuthority"] = (
        "authenticating_authority",
        [AuthenticatingAuthority],
    )
    c_cardinality["authenticating_authority"] = {"min": 0}
    c_child_order.extend(
        ["authn_context_class_ref", "authn_context_decl", "authn_context_decl_ref", "authenticating_authority"]
    )

    def __init__(
        self,
        authn_context_class_ref=None,
        authn_context_decl=None,
        authn_context_decl_ref=None,
        authenticating_authority=None,
        text=None,
        extension_elements=None,
        extension_attributes=None,
    ):
        SamlBase.__init__(
            self, text=text, extension_elements=extension_elements, extension_attributes=extension_attributes
        )
        self.authn_context_class_ref = authn_context_class_ref
        self.authn_context_decl = authn_context_decl
        self.authn_context_decl_ref = authn_context_decl_ref
        self.authenticating_authority = authenticating_authority or []

    def verify(self):
        if self.authn_context_decl and self.authn_context_decl_ref:
            raise Exception("Invalid Response: " "Cannot have both <AuthnContextDecl> and <AuthnContextDeclRef>")
        return SamlBase.verify(self)


def authn_context_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(AuthnContextType_, xml_string)


class Action(ActionType_):
    """The urn:oasis:names:tc:SAML:2.0:assertion:Action element"""

    c_tag = "Action"
    c_namespace = NAMESPACE
    c_children = ActionType_.c_children.copy()
    c_attributes = ActionType_.c_attributes.copy()
    c_child_order = ActionType_.c_child_order[:]
    c_cardinality = ActionType_.c_cardinality.copy()


def action_from_string(xml_string):
    return saml2.create_class_from_xml_string(Action, xml_string)


class AttributeType_(SamlBase):
    """The urn:oasis:names:tc:SAML:2.0:assertion:AttributeType element"""

    c_tag = "AttributeType"
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_children["{urn:oasis:names:tc:SAML:2.0:assertion}AttributeValue"] = ("attribute_value", [AttributeValue])
    c_cardinality["attribute_value"] = {"min": 0}
    c_attributes["Name"] = ("name", "string", True)
    c_attributes["NameFormat"] = ("name_format", "anyURI", False)
    c_attributes["FriendlyName"] = ("friendly_name", "string", False)
    c_child_order.extend(["attribute_value"])
    c_any_attribute = {"namespace": "##other", "processContents": "lax"}

    def __init__(
        self,
        attribute_value=None,
        name=None,
        name_format=NAME_FORMAT_URI,
        friendly_name=None,
        text=None,
        extension_elements=None,
        extension_attributes=None,
    ):
        SamlBase.__init__(
            self, text=text, extension_elements=extension_elements, extension_attributes=extension_attributes
        )
        self.attribute_value = attribute_value or []
        self.name = name
        self.name_format = name_format
        self.friendly_name = friendly_name

    # when consuming such elements, default to NAME_FORMAT_UNSPECIFIED as NameFormat
    def harvest_element_tree(self, tree):
        tree.attrib.setdefault("NameFormat", NAME_FORMAT_UNSPECIFIED)
        SamlBase.harvest_element_tree(self, tree)


def attribute_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(AttributeType_, xml_string)


class SubjectConfirmationType_(SamlBase):
    """The urn:oasis:names:tc:SAML:2.0:assertion:SubjectConfirmationType
    element"""

    c_tag = "SubjectConfirmationType"
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_children["{urn:oasis:names:tc:SAML:2.0:assertion}BaseID"] = ("base_id", BaseID)
    c_cardinality["base_id"] = {"min": 0, "max": 1}
    c_children["{urn:oasis:names:tc:SAML:2.0:assertion}NameID"] = ("name_id", NameID)
    c_cardinality["name_id"] = {"min": 0, "max": 1}
    c_children["{urn:oasis:names:tc:SAML:2.0:assertion}EncryptedID"] = ("encrypted_id", EncryptedID)
    c_cardinality["encrypted_id"] = {"min": 0, "max": 1}
    c_children["{urn:oasis:names:tc:SAML:2.0:assertion}SubjectConfirmationData"] = (
        "subject_confirmation_data",
        SubjectConfirmationData,
    )
    c_cardinality["subject_confirmation_data"] = {"min": 0, "max": 1}
    c_attributes["Method"] = ("method", "anyURI", True)
    c_child_order.extend(["base_id", "name_id", "encrypted_id", "subject_confirmation_data"])

    def __init__(
        self,
        base_id=None,
        name_id=None,
        encrypted_id=None,
        subject_confirmation_data=None,
        method=None,
        text=None,
        extension_elements=None,
        extension_attributes=None,
    ):
        SamlBase.__init__(
            self, text=text, extension_elements=extension_elements, extension_attributes=extension_attributes
        )
        self.base_id = base_id
        self.name_id = name_id
        self.encrypted_id = encrypted_id
        self.subject_confirmation_data = subject_confirmation_data
        self.method = method


def subject_confirmation_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(SubjectConfirmationType_, xml_string)


class AudienceRestriction(AudienceRestrictionType_):
    """The urn:oasis:names:tc:SAML:2.0:assertion:AudienceRestriction element"""

    c_tag = "AudienceRestriction"
    c_namespace = NAMESPACE
    c_children = AudienceRestrictionType_.c_children.copy()
    c_attributes = AudienceRestrictionType_.c_attributes.copy()
    c_child_order = AudienceRestrictionType_.c_child_order[:]
    c_cardinality = AudienceRestrictionType_.c_cardinality.copy()


def audience_restriction_from_string(xml_string):
    return saml2.create_class_from_xml_string(AudienceRestriction, xml_string)


class AuthnContext(AuthnContextType_):
    """The urn:oasis:names:tc:SAML:2.0:assertion:AuthnContext element"""

    c_tag = "AuthnContext"
    c_namespace = NAMESPACE
    c_children = AuthnContextType_.c_children.copy()
    c_attributes = AuthnContextType_.c_attributes.copy()
    c_child_order = AuthnContextType_.c_child_order[:]
    c_cardinality = AuthnContextType_.c_cardinality.copy()


def authn_context_from_string(xml_string):
    return saml2.create_class_from_xml_string(AuthnContext, xml_string)


class Attribute(AttributeType_):
    """The urn:oasis:names:tc:SAML:2.0:assertion:Attribute element"""

    c_tag = "Attribute"
    c_namespace = NAMESPACE
    c_children = AttributeType_.c_children.copy()
    c_attributes = AttributeType_.c_attributes.copy()
    c_child_order = AttributeType_.c_child_order[:]
    c_cardinality = AttributeType_.c_cardinality.copy()


def attribute_from_string(xml_string):
    return saml2.create_class_from_xml_string(Attribute, xml_string)


class SubjectConfirmation(SubjectConfirmationType_):
    """The urn:oasis:names:tc:SAML:2.0:assertion:SubjectConfirmation element"""

    c_tag = "SubjectConfirmation"
    c_namespace = NAMESPACE
    c_children = SubjectConfirmationType_.c_children.copy()
    c_attributes = SubjectConfirmationType_.c_attributes.copy()
    c_child_order = SubjectConfirmationType_.c_child_order[:]
    c_cardinality = SubjectConfirmationType_.c_cardinality.copy()


def subject_confirmation_from_string(xml_string):
    return saml2.create_class_from_xml_string(SubjectConfirmation, xml_string)


class ConditionsType_(SamlBase):
    """The urn:oasis:names:tc:SAML:2.0:assertion:ConditionsType element"""

    c_tag = "ConditionsType"
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_children["{urn:oasis:names:tc:SAML:2.0:assertion}Condition"] = ("condition", [Condition])
    c_cardinality["condition"] = {"min": 0}
    c_children["{urn:oasis:names:tc:SAML:2.0:assertion}AudienceRestriction"] = (
        "audience_restriction",
        [AudienceRestriction],
    )
    c_cardinality["audience_restriction"] = {"min": 0}
    c_children["{urn:oasis:names:tc:SAML:2.0:assertion}OneTimeUse"] = ("one_time_use", [OneTimeUse])
    c_cardinality["one_time_use"] = {"min": 0}
    c_children["{urn:oasis:names:tc:SAML:2.0:assertion}ProxyRestriction"] = ("proxy_restriction", [ProxyRestriction])
    c_cardinality["proxy_restriction"] = {"min": 0}
    c_attributes["NotBefore"] = ("not_before", "dateTime", False)
    c_attributes["NotOnOrAfter"] = ("not_on_or_after", "dateTime", False)
    c_child_order.extend(["condition", "audience_restriction", "one_time_use", "proxy_restriction"])

    def __init__(
        self,
        condition=None,
        audience_restriction=None,
        one_time_use=None,
        proxy_restriction=None,
        not_before=None,
        not_on_or_after=None,
        text=None,
        extension_elements=None,
        extension_attributes=None,
    ):
        SamlBase.__init__(
            self, text=text, extension_elements=extension_elements, extension_attributes=extension_attributes
        )
        self.condition = condition or []
        self.audience_restriction = audience_restriction or []
        self.one_time_use = one_time_use or []
        self.proxy_restriction = proxy_restriction or []
        self.not_before = not_before
        self.not_on_or_after = not_on_or_after

    def verify(self):
        if self.one_time_use:
            if len(self.one_time_use) != 1:
                raise Exception("Cannot be used more than once")
        if self.proxy_restriction:
            if len(self.proxy_restriction) != 1:
                raise Exception("Cannot be used more than once")

        return SamlBase.verify(self)


def conditions_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(ConditionsType_, xml_string)


class AuthnStatementType_(StatementAbstractType_):
    """The urn:oasis:names:tc:SAML:2.0:assertion:AuthnStatementType element"""

    c_tag = "AuthnStatementType"
    c_namespace = NAMESPACE
    c_children = StatementAbstractType_.c_children.copy()
    c_attributes = StatementAbstractType_.c_attributes.copy()
    c_child_order = StatementAbstractType_.c_child_order[:]
    c_cardinality = StatementAbstractType_.c_cardinality.copy()
    c_children["{urn:oasis:names:tc:SAML:2.0:assertion}SubjectLocality"] = ("subject_locality", SubjectLocality)
    c_cardinality["subject_locality"] = {"min": 0, "max": 1}
    c_children["{urn:oasis:names:tc:SAML:2.0:assertion}AuthnContext"] = ("authn_context", AuthnContext)
    c_attributes["AuthnInstant"] = ("authn_instant", "dateTime", True)
    c_attributes["SessionIndex"] = ("session_index", "string", False)
    c_attributes["SessionNotOnOrAfter"] = ("session_not_on_or_after", "dateTime", False)
    c_child_order.extend(["subject_locality", "authn_context"])

    def __init__(
        self,
        subject_locality=None,
        authn_context=None,
        authn_instant=None,
        session_index=None,
        session_not_on_or_after=None,
        text=None,
        extension_elements=None,
        extension_attributes=None,
    ):
        StatementAbstractType_.__init__(
            self, text=text, extension_elements=extension_elements, extension_attributes=extension_attributes
        )
        self.subject_locality = subject_locality
        self.authn_context = authn_context
        self.authn_instant = authn_instant
        self.session_index = session_index
        self.session_not_on_or_after = session_not_on_or_after


def authn_statement_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(AuthnStatementType_, xml_string)


class AttributeStatementType_(StatementAbstractType_):
    """The urn:oasis:names:tc:SAML:2.0:assertion:AttributeStatementType
    element"""

    c_tag = "AttributeStatementType"
    c_namespace = NAMESPACE
    c_children = StatementAbstractType_.c_children.copy()
    c_attributes = StatementAbstractType_.c_attributes.copy()
    c_child_order = StatementAbstractType_.c_child_order[:]
    c_cardinality = StatementAbstractType_.c_cardinality.copy()
    c_children["{urn:oasis:names:tc:SAML:2.0:assertion}Attribute"] = ("attribute", [Attribute])
    c_cardinality["attribute"] = {"min": 0}
    c_children["{urn:oasis:names:tc:SAML:2.0:assertion}EncryptedAttribute"] = (
        "encrypted_attribute",
        [EncryptedAttribute],
    )
    c_cardinality["encrypted_attribute"] = {"min": 0}
    c_child_order.extend(["attribute", "encrypted_attribute"])

    def __init__(
        self, attribute=None, encrypted_attribute=None, text=None, extension_elements=None, extension_attributes=None
    ):
        StatementAbstractType_.__init__(
            self, text=text, extension_elements=extension_elements, extension_attributes=extension_attributes
        )
        self.attribute = attribute or []
        self.encrypted_attribute = encrypted_attribute or []


def attribute_statement_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(AttributeStatementType_, xml_string)


class SubjectType_(SamlBase):
    """The urn:oasis:names:tc:SAML:2.0:assertion:SubjectType element"""

    c_tag = "SubjectType"
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_children["{urn:oasis:names:tc:SAML:2.0:assertion}BaseID"] = ("base_id", BaseID)
    c_cardinality["base_id"] = {"min": 0, "max": 1}
    c_children["{urn:oasis:names:tc:SAML:2.0:assertion}NameID"] = ("name_id", NameID)
    c_cardinality["name_id"] = {"min": 0, "max": 1}
    c_children["{urn:oasis:names:tc:SAML:2.0:assertion}EncryptedID"] = ("encrypted_id", EncryptedID)
    c_cardinality["encrypted_id"] = {"min": 0, "max": 1}
    c_children["{urn:oasis:names:tc:SAML:2.0:assertion}SubjectConfirmation"] = (
        "subject_confirmation",
        [SubjectConfirmation],
    )
    c_cardinality["subject_confirmation"] = {"min": 0}
    c_child_order.extend(["base_id", "name_id", "encrypted_id", "subject_confirmation"])

    def __init__(
        self,
        base_id=None,
        name_id=None,
        encrypted_id=None,
        subject_confirmation=None,
        text=None,
        extension_elements=None,
        extension_attributes=None,
    ):
        SamlBase.__init__(
            self, text=text, extension_elements=extension_elements, extension_attributes=extension_attributes
        )
        self.base_id = base_id
        self.name_id = name_id
        self.encrypted_id = encrypted_id
        self.subject_confirmation = subject_confirmation or []


def subject_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(SubjectType_, xml_string)


class Conditions(ConditionsType_):
    """The urn:oasis:names:tc:SAML:2.0:assertion:Conditions element"""

    c_tag = "Conditions"
    c_namespace = NAMESPACE
    c_children = ConditionsType_.c_children.copy()
    c_attributes = ConditionsType_.c_attributes.copy()
    c_child_order = ConditionsType_.c_child_order[:]
    c_cardinality = ConditionsType_.c_cardinality.copy()


def conditions_from_string(xml_string):
    return saml2.create_class_from_xml_string(Conditions, xml_string)


class AuthnStatement(AuthnStatementType_):
    """The urn:oasis:names:tc:SAML:2.0:assertion:AuthnStatement element"""

    c_tag = "AuthnStatement"
    c_namespace = NAMESPACE
    c_children = AuthnStatementType_.c_children.copy()
    c_attributes = AuthnStatementType_.c_attributes.copy()
    c_child_order = AuthnStatementType_.c_child_order[:]
    c_cardinality = AuthnStatementType_.c_cardinality.copy()


def authn_statement_from_string(xml_string):
    return saml2.create_class_from_xml_string(AuthnStatement, xml_string)


class AttributeStatement(AttributeStatementType_):
    """The urn:oasis:names:tc:SAML:2.0:assertion:AttributeStatement element"""

    c_tag = "AttributeStatement"
    c_namespace = NAMESPACE
    c_children = AttributeStatementType_.c_children.copy()
    c_attributes = AttributeStatementType_.c_attributes.copy()
    c_child_order = AttributeStatementType_.c_child_order[:]
    c_cardinality = AttributeStatementType_.c_cardinality.copy()


def attribute_statement_from_string(xml_string):
    return saml2.create_class_from_xml_string(AttributeStatement, xml_string)


class Subject(SubjectType_):
    """The urn:oasis:names:tc:SAML:2.0:assertion:Subject element"""

    c_tag = "Subject"
    c_namespace = NAMESPACE
    c_children = SubjectType_.c_children.copy()
    c_attributes = SubjectType_.c_attributes.copy()
    c_child_order = SubjectType_.c_child_order[:]
    c_cardinality = SubjectType_.c_cardinality.copy()


def subject_from_string(xml_string):
    return saml2.create_class_from_xml_string(Subject, xml_string)


# ..................
# ['AuthzDecisionStatement', 'EvidenceType', 'AdviceType', 'Evidence',
# 'Assertion', 'AssertionType', 'AuthzDecisionStatementType', 'Advice']
class EvidenceType_(SamlBase):
    """The urn:oasis:names:tc:SAML:2.0:assertion:EvidenceType element"""

    c_tag = "EvidenceType"
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_children["{urn:oasis:names:tc:SAML:2.0:assertion}AssertionIDRef"] = ("assertion_id_ref", [AssertionIDRef])
    c_cardinality["assertion_id_ref"] = {"min": 0}
    c_children["{urn:oasis:names:tc:SAML:2.0:assertion}AssertionURIRef"] = ("assertion_uri_ref", [AssertionURIRef])
    c_cardinality["assertion_uri_ref"] = {"min": 0}
    c_cardinality["assertion"] = {"min": 0}
    c_children["{urn:oasis:names:tc:SAML:2.0:assertion}EncryptedAssertion"] = (
        "encrypted_assertion",
        [EncryptedAssertion],
    )
    c_cardinality["encrypted_assertion"] = {"min": 0}
    c_child_order.extend(["assertion_id_ref", "assertion_uri_ref", "assertion", "encrypted_assertion"])

    def __init__(
        self,
        assertion_id_ref=None,
        assertion_uri_ref=None,
        assertion=None,
        encrypted_assertion=None,
        text=None,
        extension_elements=None,
        extension_attributes=None,
    ):
        SamlBase.__init__(
            self, text=text, extension_elements=extension_elements, extension_attributes=extension_attributes
        )
        self.assertion_id_ref = assertion_id_ref or []
        self.assertion_uri_ref = assertion_uri_ref or []
        self.assertion = assertion or []
        self.encrypted_assertion = encrypted_assertion or []


def evidence_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(EvidenceType_, xml_string)


class Evidence(EvidenceType_):
    """The urn:oasis:names:tc:SAML:2.0:assertion:Evidence element"""

    c_tag = "Evidence"
    c_namespace = NAMESPACE
    c_children = EvidenceType_.c_children.copy()
    c_attributes = EvidenceType_.c_attributes.copy()
    c_child_order = EvidenceType_.c_child_order[:]
    c_cardinality = EvidenceType_.c_cardinality.copy()


def evidence_from_string(xml_string):
    return saml2.create_class_from_xml_string(Evidence, xml_string)


class AuthzDecisionStatementType_(StatementAbstractType_):
    """The urn:oasis:names:tc:SAML:2.0:assertion:AuthzDecisionStatementType
    element"""

    c_tag = "AuthzDecisionStatementType"
    c_namespace = NAMESPACE
    c_children = StatementAbstractType_.c_children.copy()
    c_attributes = StatementAbstractType_.c_attributes.copy()
    c_child_order = StatementAbstractType_.c_child_order[:]
    c_cardinality = StatementAbstractType_.c_cardinality.copy()
    c_children["{urn:oasis:names:tc:SAML:2.0:assertion}Action"] = ("action", [Action])
    c_cardinality["action"] = {"min": 1}
    c_children["{urn:oasis:names:tc:SAML:2.0:assertion}Evidence"] = ("evidence", Evidence)
    c_cardinality["evidence"] = {"min": 0, "max": 1}
    c_attributes["Resource"] = ("resource", "anyURI", True)
    c_attributes["Decision"] = ("decision", DecisionType_, True)
    c_child_order.extend(["action", "evidence"])

    def __init__(
        self,
        action=None,
        evidence=None,
        resource=None,
        decision=None,
        text=None,
        extension_elements=None,
        extension_attributes=None,
    ):
        StatementAbstractType_.__init__(
            self, text=text, extension_elements=extension_elements, extension_attributes=extension_attributes
        )
        self.action = action or []
        self.evidence = evidence
        self.resource = resource
        self.decision = decision


def authz_decision_statement_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(AuthzDecisionStatementType_, xml_string)


class AuthzDecisionStatement(AuthzDecisionStatementType_):
    """The urn:oasis:names:tc:SAML:2.0:assertion:AuthzDecisionStatement
    element"""

    c_tag = "AuthzDecisionStatement"
    c_namespace = NAMESPACE
    c_children = AuthzDecisionStatementType_.c_children.copy()
    c_attributes = AuthzDecisionStatementType_.c_attributes.copy()
    c_child_order = AuthzDecisionStatementType_.c_child_order[:]
    c_cardinality = AuthzDecisionStatementType_.c_cardinality.copy()


def authz_decision_statement_from_string(xml_string):
    return saml2.create_class_from_xml_string(AuthzDecisionStatement, xml_string)


# ..................
# ['Assertion', 'AssertionType', 'AdviceType', 'Advice']
class AssertionType_(SamlBase):
    """The urn:oasis:names:tc:SAML:2.0:assertion:AssertionType element"""

    c_tag = "AssertionType"
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_children["{urn:oasis:names:tc:SAML:2.0:assertion}Issuer"] = ("issuer", Issuer)
    c_children["{http://www.w3.org/2000/09/xmldsig#}Signature"] = ("signature", ds.Signature)
    c_cardinality["signature"] = {"min": 0, "max": 1}
    c_children["{urn:oasis:names:tc:SAML:2.0:assertion}Subject"] = ("subject", Subject)
    c_cardinality["subject"] = {"min": 0, "max": 1}
    c_children["{urn:oasis:names:tc:SAML:2.0:assertion}Conditions"] = ("conditions", Conditions)
    c_cardinality["conditions"] = {"min": 0, "max": 1}
    c_cardinality["advice"] = {"min": 0, "max": 1}
    c_children["{urn:oasis:names:tc:SAML:2.0:assertion}Statement"] = ("statement", [Statement])
    c_cardinality["statement"] = {"min": 0}
    c_children["{urn:oasis:names:tc:SAML:2.0:assertion}AuthnStatement"] = ("authn_statement", [AuthnStatement])
    c_cardinality["authn_statement"] = {"min": 0}
    c_children["{urn:oasis:names:tc:SAML:2.0:assertion}AuthzDecisionStatement"] = (
        "authz_decision_statement",
        [AuthzDecisionStatement],
    )
    c_cardinality["authz_decision_statement"] = {"min": 0}
    c_children["{urn:oasis:names:tc:SAML:2.0:assertion}AttributeStatement"] = (
        "attribute_statement",
        [AttributeStatement],
    )
    c_cardinality["attribute_statement"] = {"min": 0}
    c_attributes["Version"] = ("version", "string", True)
    c_attributes["ID"] = ("id", "ID", True)
    c_attributes["IssueInstant"] = ("issue_instant", "dateTime", True)
    c_child_order.extend(
        [
            "issuer",
            "signature",
            "subject",
            "conditions",
            "advice",
            "statement",
            "authn_statement",
            "authz_decision_statement",
            "attribute_statement",
        ]
    )

    def __init__(
        self,
        issuer=None,
        signature=None,
        subject=None,
        conditions=None,
        advice=None,
        statement=None,
        authn_statement=None,
        authz_decision_statement=None,
        attribute_statement=None,
        version=None,
        id=None,
        issue_instant=None,
        text=None,
        extension_elements=None,
        extension_attributes=None,
    ):
        SamlBase.__init__(
            self, text=text, extension_elements=extension_elements, extension_attributes=extension_attributes
        )
        self.issuer = issuer
        self.signature = signature
        self.subject = subject
        self.conditions = conditions
        self.advice = advice
        self.statement = statement or []
        self.authn_statement = authn_statement or []
        self.authz_decision_statement = authz_decision_statement or []
        self.attribute_statement = attribute_statement or []
        self.version = version
        self.id = id
        self.issue_instant = issue_instant

    def verify(self):
        # If no statement MUST contain a subject element
        if self.attribute_statement or self.statement or self.authn_statement or self.authz_decision_statement:
            pass
        elif not self.subject:
            raise MustValueError("If no statement MUST contain a subject element")

        if self.authn_statement and not self.subject:
            raise MustValueError("An assertion with an AuthnStatement must contain a Subject")

        return SamlBase.verify(self)


def assertion_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(AssertionType_, xml_string)


class Assertion(AssertionType_):
    """The urn:oasis:names:tc:SAML:2.0:assertion:Assertion element"""

    c_tag = "Assertion"
    c_namespace = NAMESPACE
    c_children = AssertionType_.c_children.copy()
    c_attributes = AssertionType_.c_attributes.copy()
    c_child_order = AssertionType_.c_child_order[:]
    c_cardinality = AssertionType_.c_cardinality.copy()


def assertion_from_string(xml_string):
    return saml2.create_class_from_xml_string(Assertion, xml_string)


class AdviceType_(SamlBase):
    """The urn:oasis:names:tc:SAML:2.0:assertion:AdviceType element"""

    c_tag = "AdviceType"
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_children["{urn:oasis:names:tc:SAML:2.0:assertion}AssertionIDRef"] = ("assertion_id_ref", [AssertionIDRef])
    c_cardinality["assertion_id_ref"] = {"min": 0}
    c_children["{urn:oasis:names:tc:SAML:2.0:assertion}AssertionURIRef"] = ("assertion_uri_ref", [AssertionURIRef])
    c_cardinality["assertion_uri_ref"] = {"min": 0}
    c_children["{urn:oasis:names:tc:SAML:2.0:assertion}Assertion"] = ("assertion", [Assertion])
    c_cardinality["assertion"] = {"min": 0}
    c_children["{urn:oasis:names:tc:SAML:2.0:assertion}EncryptedAssertion"] = (
        "encrypted_assertion",
        [EncryptedAssertion],
    )
    c_cardinality["encrypted_assertion"] = {"min": 0}
    c_child_order.extend(["assertion_id_ref", "assertion_uri_ref", "assertion", "encrypted_assertion"])
    c_any = {"namespace": "##other", "processContents": "lax"}

    def __init__(
        self,
        assertion_id_ref=None,
        assertion_uri_ref=None,
        assertion=None,
        encrypted_assertion=None,
        text=None,
        extension_elements=None,
        extension_attributes=None,
    ):
        SamlBase.__init__(
            self, text=text, extension_elements=extension_elements, extension_attributes=extension_attributes
        )
        self.assertion_id_ref = assertion_id_ref or []
        self.assertion_uri_ref = assertion_uri_ref or []
        self.assertion = assertion or []
        self.encrypted_assertion = encrypted_assertion or []


def advice_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(AdviceType_, xml_string)


class Advice(AdviceType_):
    """The urn:oasis:names:tc:SAML:2.0:assertion:Advice element"""

    c_tag = "Advice"
    c_namespace = NAMESPACE
    c_children = AdviceType_.c_children.copy()
    c_attributes = AdviceType_.c_attributes.copy()
    c_child_order = AdviceType_.c_child_order[:]
    c_cardinality = AdviceType_.c_cardinality.copy()


def advice_from_string(xml_string):
    return saml2.create_class_from_xml_string(Advice, xml_string)


# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
EvidenceType_.c_children["{urn:oasis:names:tc:SAML:2.0:assertion}Assertion"] = ("assertion", [Assertion])
Evidence.c_children["{urn:oasis:names:tc:SAML:2.0:assertion}Assertion"] = ("assertion", [Assertion])
AssertionType_.c_children["{urn:oasis:names:tc:SAML:2.0:assertion}Advice"] = ("advice", Advice)
Assertion.c_children["{urn:oasis:names:tc:SAML:2.0:assertion}Advice"] = ("advice", Advice)
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

AG_IDNameQualifiers = [
    ("NameQualifier", "string", False),
    ("SPNameQualifier", "string", False),
]

ELEMENT_FROM_STRING = {
    BaseID.c_tag: base_id_from_string,
    NameID.c_tag: name_id_from_string,
    NameIDType_.c_tag: name_id_type__from_string,
    EncryptedElementType_.c_tag: encrypted_element_type__from_string,
    EncryptedID.c_tag: encrypted_id_from_string,
    Issuer.c_tag: issuer_from_string,
    AssertionIDRef.c_tag: assertion_id_ref_from_string,
    AssertionURIRef.c_tag: assertion_uri_ref_from_string,
    Assertion.c_tag: assertion_from_string,
    AssertionType_.c_tag: assertion_type__from_string,
    Subject.c_tag: subject_from_string,
    SubjectType_.c_tag: subject_type__from_string,
    SubjectConfirmation.c_tag: subject_confirmation_from_string,
    SubjectConfirmationType_.c_tag: subject_confirmation_type__from_string,
    SubjectConfirmationData.c_tag: subject_confirmation_data_from_string,
    SubjectConfirmationDataType_.c_tag: subject_confirmation_data_type__from_string,
    KeyInfoConfirmationDataType_.c_tag: key_info_confirmation_data_type__from_string,
    Conditions.c_tag: conditions_from_string,
    ConditionsType_.c_tag: conditions_type__from_string,
    Condition.c_tag: condition_from_string,
    AudienceRestriction.c_tag: audience_restriction_from_string,
    AudienceRestrictionType_.c_tag: audience_restriction_type__from_string,
    Audience.c_tag: audience_from_string,
    OneTimeUse.c_tag: one_time_use_from_string,
    OneTimeUseType_.c_tag: one_time_use_type__from_string,
    ProxyRestriction.c_tag: proxy_restriction_from_string,
    ProxyRestrictionType_.c_tag: proxy_restriction_type__from_string,
    Advice.c_tag: advice_from_string,
    AdviceType_.c_tag: advice_type__from_string,
    EncryptedAssertion.c_tag: encrypted_assertion_from_string,
    Statement.c_tag: statement_from_string,
    AuthnStatement.c_tag: authn_statement_from_string,
    AuthnStatementType_.c_tag: authn_statement_type__from_string,
    SubjectLocality.c_tag: subject_locality_from_string,
    SubjectLocalityType_.c_tag: subject_locality_type__from_string,
    AuthnContext.c_tag: authn_context_from_string,
    AuthnContextType_.c_tag: authn_context_type__from_string,
    AuthnContextClassRef.c_tag: authn_context_class_ref_from_string,
    AuthnContextDeclRef.c_tag: authn_context_decl_ref_from_string,
    AuthnContextDecl.c_tag: authn_context_decl_from_string,
    AuthenticatingAuthority.c_tag: authenticating_authority_from_string,
    AuthzDecisionStatement.c_tag: authz_decision_statement_from_string,
    AuthzDecisionStatementType_.c_tag: authz_decision_statement_type__from_string,
    DecisionType_.c_tag: decision_type__from_string,
    Action.c_tag: action_from_string,
    ActionType_.c_tag: action_type__from_string,
    Evidence.c_tag: evidence_from_string,
    EvidenceType_.c_tag: evidence_type__from_string,
    AttributeStatement.c_tag: attribute_statement_from_string,
    AttributeStatementType_.c_tag: attribute_statement_type__from_string,
    Attribute.c_tag: attribute_from_string,
    AttributeType_.c_tag: attribute_type__from_string,
    AttributeValue.c_tag: attribute_value_from_string,
    EncryptedAttribute.c_tag: encrypted_attribute_from_string,
}

ELEMENT_BY_TAG = {
    "BaseID": BaseID,
    "NameID": NameID,
    "NameIDType": NameIDType_,
    "EncryptedElementType": EncryptedElementType_,
    "EncryptedID": EncryptedID,
    "Issuer": Issuer,
    "AssertionIDRef": AssertionIDRef,
    "AssertionURIRef": AssertionURIRef,
    "Assertion": Assertion,
    "AssertionType": AssertionType_,
    "Subject": Subject,
    "SubjectType": SubjectType_,
    "SubjectConfirmation": SubjectConfirmation,
    "SubjectConfirmationType": SubjectConfirmationType_,
    "SubjectConfirmationData": SubjectConfirmationData,
    "SubjectConfirmationDataType": SubjectConfirmationDataType_,
    "KeyInfoConfirmationDataType": KeyInfoConfirmationDataType_,
    "Conditions": Conditions,
    "ConditionsType": ConditionsType_,
    "Condition": Condition,
    "AudienceRestriction": AudienceRestriction,
    "AudienceRestrictionType": AudienceRestrictionType_,
    "Audience": Audience,
    "OneTimeUse": OneTimeUse,
    "OneTimeUseType": OneTimeUseType_,
    "ProxyRestriction": ProxyRestriction,
    "ProxyRestrictionType": ProxyRestrictionType_,
    "Advice": Advice,
    "AdviceType": AdviceType_,
    "EncryptedAssertion": EncryptedAssertion,
    "Statement": Statement,
    "AuthnStatement": AuthnStatement,
    "AuthnStatementType": AuthnStatementType_,
    "SubjectLocality": SubjectLocality,
    "SubjectLocalityType": SubjectLocalityType_,
    "AuthnContext": AuthnContext,
    "AuthnContextType": AuthnContextType_,
    "AuthnContextClassRef": AuthnContextClassRef,
    "AuthnContextDeclRef": AuthnContextDeclRef,
    "AuthnContextDecl": AuthnContextDecl,
    "AuthenticatingAuthority": AuthenticatingAuthority,
    "AuthzDecisionStatement": AuthzDecisionStatement,
    "AuthzDecisionStatementType": AuthzDecisionStatementType_,
    "DecisionType": DecisionType_,
    "Action": Action,
    "ActionType": ActionType_,
    "Evidence": Evidence,
    "EvidenceType": EvidenceType_,
    "AttributeStatement": AttributeStatement,
    "AttributeStatementType": AttributeStatementType_,
    "Attribute": Attribute,
    "AttributeType": AttributeType_,
    "AttributeValue": AttributeValue,
    "EncryptedAttribute": EncryptedAttribute,
    "BaseIDAbstractType": BaseIDAbstractType_,
    "ConditionAbstractType": ConditionAbstractType_,
    "StatementAbstractType": StatementAbstractType_,
}


def factory(tag, **kwargs):
    return ELEMENT_BY_TAG[tag](**kwargs)



#!/usr/bin/env python

#
# Generated Sun Jun 14 12:18:10 2015 by parse_xsd.py version 0.5.
#

import saml2
from saml2 import SamlBase


NAMESPACE = "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd"


class TTimestampFault_(SamlBase):
    """The http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd:tTimestampFault element"""

    c_tag = "tTimestampFault"
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()


def t_timestamp_fault__from_string(xml_string):
    return saml2.create_class_from_xml_string(TTimestampFault_, xml_string)


class AttributedDateTime_(SamlBase):
    """The http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd:AttributedDateTime element"""

    c_tag = "AttributedDateTime"
    c_namespace = NAMESPACE
    c_value_type = {"base": "string"}
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_attributes["Id"] = ("Id", "anyURI", False)

    def __init__(
        self,
        Id=None,
        text=None,
        extension_elements=None,
        extension_attributes=None,
    ):
        SamlBase.__init__(
            self,
            text=text,
            extension_elements=extension_elements,
            extension_attributes=extension_attributes,
        )
        self.Id = Id


def attributed_date_time__from_string(xml_string):
    return saml2.create_class_from_xml_string(AttributedDateTime_, xml_string)


class AttributedURI_(SamlBase):
    """The http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd:AttributedURI element"""

    c_tag = "AttributedURI"
    c_namespace = NAMESPACE
    c_value_type = {"base": "anyURI"}
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_attributes["Id"] = ("Id", "anyURI", False)

    def __init__(
        self,
        Id=None,
        text=None,
        extension_elements=None,
        extension_attributes=None,
    ):
        SamlBase.__init__(
            self,
            text=text,
            extension_elements=extension_elements,
            extension_attributes=extension_attributes,
        )
        self.Id = Id


def attributed_ur_i__from_string(xml_string):
    return saml2.create_class_from_xml_string(AttributedURI_, xml_string)


class Expires(AttributedDateTime_):
    """The http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd:Expires element"""

    c_tag = "Expires"
    c_namespace = NAMESPACE
    c_children = AttributedDateTime_.c_children.copy()
    c_attributes = AttributedDateTime_.c_attributes.copy()
    c_child_order = AttributedDateTime_.c_child_order[:]
    c_cardinality = AttributedDateTime_.c_cardinality.copy()


def expires_from_string(xml_string):
    return saml2.create_class_from_xml_string(Expires, xml_string)


class Created(AttributedDateTime_):
    """The http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd:Created element"""

    c_tag = "Created"
    c_namespace = NAMESPACE
    c_children = AttributedDateTime_.c_children.copy()
    c_attributes = AttributedDateTime_.c_attributes.copy()
    c_child_order = AttributedDateTime_.c_child_order[:]
    c_cardinality = AttributedDateTime_.c_cardinality.copy()


def created_from_string(xml_string):
    return saml2.create_class_from_xml_string(Created, xml_string)


class TimestampType_(SamlBase):
    """The http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd:TimestampType element"""

    c_tag = "TimestampType"
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_children["{http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd}Created"] = (
        "created",
        Created,
    )
    c_cardinality["created"] = {"min": 0, "max": 1}
    c_children["{http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd}Expires"] = (
        "expires",
        Expires,
    )
    c_cardinality["expires"] = {"min": 0, "max": 1}
    c_attributes["Id"] = ("Id", "anyURI", False)
    c_child_order.extend(["created", "expires"])

    def __init__(
        self,
        created=None,
        expires=None,
        Id=None,
        text=None,
        extension_elements=None,
        extension_attributes=None,
    ):
        SamlBase.__init__(
            self,
            text=text,
            extension_elements=extension_elements,
            extension_attributes=extension_attributes,
        )
        self.created = created
        self.expires = expires
        self.Id = Id


def timestamp_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(TimestampType_, xml_string)


class Timestamp(TimestampType_):
    """The http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd:Timestamp element"""

    c_tag = "Timestamp"
    c_namespace = NAMESPACE
    c_children = TimestampType_.c_children.copy()
    c_attributes = TimestampType_.c_attributes.copy()
    c_child_order = TimestampType_.c_child_order[:]
    c_cardinality = TimestampType_.c_cardinality.copy()


def timestamp_from_string(xml_string):
    return saml2.create_class_from_xml_string(Timestamp, xml_string)


# ..................
AG_commonAtts = [
    ("Id", "", False),
]

ELEMENT_FROM_STRING = {
    TTimestampFault_.c_tag: t_timestamp_fault__from_string,
    AttributedDateTime_.c_tag: attributed_date_time__from_string,
    AttributedURI_.c_tag: attributed_ur_i__from_string,
    TimestampType_.c_tag: timestamp_type__from_string,
    Timestamp.c_tag: timestamp_from_string,
    Expires.c_tag: expires_from_string,
    Created.c_tag: created_from_string,
}

ELEMENT_BY_TAG = {
    "tTimestampFault": TTimestampFault_,
    "AttributedDateTime": AttributedDateTime_,
    "AttributedURI": AttributedURI_,
    "TimestampType": TimestampType_,
    "Timestamp": Timestamp,
    "Expires": Expires,
    "Created": Created,
}


def factory(tag, **kwargs):
    return ELEMENT_BY_TAG[tag](**kwargs)



#!/usr/bin/env python

#
# Generated Sun Jun 14 13:29:15 2015 by parse_xsd.py version 0.5.
#

import saml2
from saml2 import SamlBase


NAMESPACE = "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd"


class AttributedString_(SamlBase):
    """The http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd:AttributedString element"""

    c_tag = "AttributedString"
    c_namespace = NAMESPACE
    c_value_type = {"base": "string"}
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_attributes["{http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd}Id"] = (
        "Id",
        "string",
        False,
    )

    def __init__(
        self,
        Id=None,
        text=None,
        extension_elements=None,
        extension_attributes=None,
    ):
        SamlBase.__init__(
            self,
            text=text,
            extension_elements=extension_elements,
            extension_attributes=extension_attributes,
        )
        self.Id = Id


def attributed_string__from_string(xml_string):
    return saml2.create_class_from_xml_string(AttributedString_, xml_string)


class PasswordString_(AttributedString_):
    """The http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd:PasswordString element"""

    c_tag = "PasswordString"
    c_namespace = NAMESPACE
    c_children = AttributedString_.c_children.copy()
    c_attributes = AttributedString_.c_attributes.copy()
    c_child_order = AttributedString_.c_child_order[:]
    c_cardinality = AttributedString_.c_cardinality.copy()
    c_attributes["Type"] = ("type", "anyURI", False)

    def __init__(
        self,
        type=None,
        Id=None,
        text=None,
        extension_elements=None,
        extension_attributes=None,
    ):
        AttributedString_.__init__(
            self,
            Id=Id,
            text=text,
            extension_elements=extension_elements,
            extension_attributes=extension_attributes,
        )
        self.type = type


def password_string__from_string(xml_string):
    return saml2.create_class_from_xml_string(PasswordString_, xml_string)


class EncodedString_(AttributedString_):
    """The http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd:EncodedString element"""

    c_tag = "EncodedString"
    c_namespace = NAMESPACE
    c_children = AttributedString_.c_children.copy()
    c_attributes = AttributedString_.c_attributes.copy()
    c_child_order = AttributedString_.c_child_order[:]
    c_cardinality = AttributedString_.c_cardinality.copy()
    c_attributes["EncodingType"] = ("encoding_type", "anyURI", False)

    def __init__(
        self,
        encoding_type=None,
        Id=None,
        text=None,
        extension_elements=None,
        extension_attributes=None,
    ):
        AttributedString_.__init__(
            self,
            Id=Id,
            text=text,
            extension_elements=extension_elements,
            extension_attributes=extension_attributes,
        )
        self.encoding_type = encoding_type


def encoded_string__from_string(xml_string):
    return saml2.create_class_from_xml_string(EncodedString_, xml_string)


class UsernameTokenType_Username(AttributedString_):

    c_tag = "Username"
    c_namespace = NAMESPACE
    c_children = AttributedString_.c_children.copy()
    c_attributes = AttributedString_.c_attributes.copy()
    c_child_order = AttributedString_.c_child_order[:]
    c_cardinality = AttributedString_.c_cardinality.copy()


def username_token_type__username_from_string(xml_string):
    return saml2.create_class_from_xml_string(UsernameTokenType_Username, xml_string)


class UsernameTokenType_(SamlBase):
    """The http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd:UsernameTokenType element"""

    c_tag = "UsernameTokenType"
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_children["{http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd}Username"] = (
        "username",
        UsernameTokenType_Username,
    )
    c_attributes["{http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd}Id"] = (
        "Id",
        "None",
        False,
    )
    c_child_order.extend(["username"])

    def __init__(
        self,
        username=None,
        Id=None,
        text=None,
        extension_elements=None,
        extension_attributes=None,
    ):
        SamlBase.__init__(
            self,
            text=text,
            extension_elements=extension_elements,
            extension_attributes=extension_attributes,
        )
        self.username = username
        self.Id = Id


def username_token_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(UsernameTokenType_, xml_string)


class BinarySecurityTokenType_(EncodedString_):
    """The http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd:BinarySecurityTokenType element"""

    c_tag = "BinarySecurityTokenType"
    c_namespace = NAMESPACE
    c_children = EncodedString_.c_children.copy()
    c_attributes = EncodedString_.c_attributes.copy()
    c_child_order = EncodedString_.c_child_order[:]
    c_cardinality = EncodedString_.c_cardinality.copy()
    c_attributes["ValueType"] = ("value_type", "anyURI", False)

    def __init__(
        self,
        value_type=None,
        encoding_type=None,
        Id=None,
        text=None,
        extension_elements=None,
        extension_attributes=None,
    ):
        EncodedString_.__init__(
            self,
            encoding_type=encoding_type,
            Id=Id,
            text=text,
            extension_elements=extension_elements,
            extension_attributes=extension_attributes,
        )
        self.value_type = value_type


def binary_security_token_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(BinarySecurityTokenType_, xml_string)


class KeyIdentifierType_(EncodedString_):
    """The http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd:KeyIdentifierType element"""

    c_tag = "KeyIdentifierType"
    c_namespace = NAMESPACE
    c_children = EncodedString_.c_children.copy()
    c_attributes = EncodedString_.c_attributes.copy()
    c_child_order = EncodedString_.c_child_order[:]
    c_cardinality = EncodedString_.c_cardinality.copy()
    c_attributes["ValueType"] = ("value_type", "anyURI", False)

    def __init__(
        self,
        value_type=None,
        encoding_type=None,
        Id=None,
        text=None,
        extension_elements=None,
        extension_attributes=None,
    ):
        EncodedString_.__init__(
            self,
            encoding_type=encoding_type,
            Id=Id,
            text=text,
            extension_elements=extension_elements,
            extension_attributes=extension_attributes,
        )
        self.value_type = value_type


def key_identifier_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(KeyIdentifierType_, xml_string)


class TUsage_(SamlBase):
    """The http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd:tUsage element"""

    c_tag = "tUsage"
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()


def t_usage__from_string(xml_string):
    return saml2.create_class_from_xml_string(TUsage_, xml_string)


class ReferenceType_(SamlBase):
    """The http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd:ReferenceType element"""

    c_tag = "ReferenceType"
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_attributes["URI"] = ("uri", "anyURI", False)
    c_attributes["ValueType"] = ("value_type", "anyURI", False)

    def __init__(
        self,
        uri=None,
        value_type=None,
        text=None,
        extension_elements=None,
        extension_attributes=None,
    ):
        SamlBase.__init__(
            self,
            text=text,
            extension_elements=extension_elements,
            extension_attributes=extension_attributes,
        )
        self.uri = uri
        self.value_type = value_type


def reference_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(ReferenceType_, xml_string)


class EmbeddedType_(SamlBase):
    """The http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd:EmbeddedType element"""

    c_tag = "EmbeddedType"
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_attributes["ValueType"] = ("value_type", "anyURI", False)

    def __init__(
        self,
        value_type=None,
        text=None,
        extension_elements=None,
        extension_attributes=None,
    ):
        SamlBase.__init__(
            self,
            text=text,
            extension_elements=extension_elements,
            extension_attributes=extension_attributes,
        )
        self.value_type = value_type


def embedded_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(EmbeddedType_, xml_string)


class SecurityTokenReferenceType_(SamlBase):
    """The http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd:SecurityTokenReferenceType element"""

    c_tag = "SecurityTokenReferenceType"
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_attributes["{http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd}Id"] = (
        "Id",
        "None",
        False,
    )
    c_attributes["Usage"] = ("Usage", "None", False)

    def __init__(
        self,
        Id=None,
        Usage=None,
        text=None,
        extension_elements=None,
        extension_attributes=None,
    ):
        SamlBase.__init__(
            self,
            text=text,
            extension_elements=extension_elements,
            extension_attributes=extension_attributes,
        )
        self.Id = Id
        self.Usage = Usage


def security_token_reference_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(SecurityTokenReferenceType_, xml_string)


class SecurityHeaderType_(SamlBase):
    """The http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd:SecurityHeaderType element"""

    c_tag = "SecurityHeaderType"
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()


def security_header_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(SecurityHeaderType_, xml_string)


class TransformationParametersType_(SamlBase):
    """The http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd:TransformationParametersType element"""

    c_tag = "TransformationParametersType"
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()


def transformation_parameters_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(TransformationParametersType_, xml_string)


class UsernameToken(UsernameTokenType_):
    """The http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd:UsernameToken element"""

    c_tag = "UsernameToken"
    c_namespace = NAMESPACE
    c_children = UsernameTokenType_.c_children.copy()
    c_attributes = UsernameTokenType_.c_attributes.copy()
    c_child_order = UsernameTokenType_.c_child_order[:]
    c_cardinality = UsernameTokenType_.c_cardinality.copy()


def username_token_from_string(xml_string):
    return saml2.create_class_from_xml_string(UsernameToken, xml_string)


class BinarySecurityToken(BinarySecurityTokenType_):
    """The http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd:BinarySecurityToken element"""

    c_tag = "BinarySecurityToken"
    c_namespace = NAMESPACE
    c_children = BinarySecurityTokenType_.c_children.copy()
    c_attributes = BinarySecurityTokenType_.c_attributes.copy()
    c_child_order = BinarySecurityTokenType_.c_child_order[:]
    c_cardinality = BinarySecurityTokenType_.c_cardinality.copy()


def binary_security_token_from_string(xml_string):
    return saml2.create_class_from_xml_string(BinarySecurityToken, xml_string)


class Reference(ReferenceType_):
    """The http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd:Reference element"""

    c_tag = "Reference"
    c_namespace = NAMESPACE
    c_children = ReferenceType_.c_children.copy()
    c_attributes = ReferenceType_.c_attributes.copy()
    c_child_order = ReferenceType_.c_child_order[:]
    c_cardinality = ReferenceType_.c_cardinality.copy()


def reference_from_string(xml_string):
    return saml2.create_class_from_xml_string(Reference, xml_string)


class Embedded(EmbeddedType_):
    """The http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd:Embedded element"""

    c_tag = "Embedded"
    c_namespace = NAMESPACE
    c_children = EmbeddedType_.c_children.copy()
    c_attributes = EmbeddedType_.c_attributes.copy()
    c_child_order = EmbeddedType_.c_child_order[:]
    c_cardinality = EmbeddedType_.c_cardinality.copy()


def embedded_from_string(xml_string):
    return saml2.create_class_from_xml_string(Embedded, xml_string)


class KeyIdentifier(KeyIdentifierType_):
    """The http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd:KeyIdentifier element"""

    c_tag = "KeyIdentifier"
    c_namespace = NAMESPACE
    c_children = KeyIdentifierType_.c_children.copy()
    c_attributes = KeyIdentifierType_.c_attributes.copy()
    c_child_order = KeyIdentifierType_.c_child_order[:]
    c_cardinality = KeyIdentifierType_.c_cardinality.copy()


def key_identifier_from_string(xml_string):
    return saml2.create_class_from_xml_string(KeyIdentifier, xml_string)


class SecurityTokenReference(SecurityTokenReferenceType_):
    """The http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd:SecurityTokenReference element"""

    c_tag = "SecurityTokenReference"
    c_namespace = NAMESPACE
    c_children = SecurityTokenReferenceType_.c_children.copy()
    c_attributes = SecurityTokenReferenceType_.c_attributes.copy()
    c_child_order = SecurityTokenReferenceType_.c_child_order[:]
    c_cardinality = SecurityTokenReferenceType_.c_cardinality.copy()


def security_token_reference_from_string(xml_string):
    return saml2.create_class_from_xml_string(SecurityTokenReference, xml_string)


class Security(SecurityHeaderType_):
    """The http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd:Security element"""

    c_tag = "Security"
    c_namespace = NAMESPACE
    c_children = SecurityHeaderType_.c_children.copy()
    c_attributes = SecurityHeaderType_.c_attributes.copy()
    c_child_order = SecurityHeaderType_.c_child_order[:]
    c_cardinality = SecurityHeaderType_.c_cardinality.copy()


def security_from_string(xml_string):
    return saml2.create_class_from_xml_string(Security, xml_string)


class TransformationParameters(TransformationParametersType_):
    """The http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd:TransformationParameters element"""

    c_tag = "TransformationParameters"
    c_namespace = NAMESPACE
    c_children = TransformationParametersType_.c_children.copy()
    c_attributes = TransformationParametersType_.c_attributes.copy()
    c_child_order = TransformationParametersType_.c_child_order[:]
    c_cardinality = TransformationParametersType_.c_cardinality.copy()


def transformation_parameters_from_string(xml_string):
    return saml2.create_class_from_xml_string(TransformationParameters, xml_string)


class Password(PasswordString_):
    """The http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd:Password element"""

    c_tag = "Password"
    c_namespace = NAMESPACE
    c_children = PasswordString_.c_children.copy()
    c_attributes = PasswordString_.c_attributes.copy()
    c_child_order = PasswordString_.c_child_order[:]
    c_cardinality = PasswordString_.c_cardinality.copy()


def password_from_string(xml_string):
    return saml2.create_class_from_xml_string(Password, xml_string)


class Nonce(EncodedString_):
    """The http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd:Nonce element"""

    c_tag = "Nonce"
    c_namespace = NAMESPACE
    c_children = EncodedString_.c_children.copy()
    c_attributes = EncodedString_.c_attributes.copy()
    c_child_order = EncodedString_.c_child_order[:]
    c_cardinality = EncodedString_.c_cardinality.copy()


def nonce_from_string(xml_string):
    return saml2.create_class_from_xml_string(Nonce, xml_string)


class FaultcodeEnum_(SamlBase):
    """The http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd:FaultcodeEnum element"""

    c_tag = "FaultcodeEnum"
    c_namespace = NAMESPACE
    c_value_type = {
        "base": "xsd:QName",
        "enumeration": [
            "wsse:UnsupportedSecurityToken",
            "wsse:UnsupportedAlgorithm",
            "wsse:InvalidSecurity",
            "wsse:InvalidSecurityToken",
            "wsse:FailedAuthentication",
            "wsse:FailedCheck",
            "wsse:SecurityTokenUnavailable",
        ],
    }
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()


def faultcode_enum__from_string(xml_string):
    return saml2.create_class_from_xml_string(FaultcodeEnum_, xml_string)


# ..................
("#", [])
ELEMENT_FROM_STRING = {
    AttributedString_.c_tag: attributed_string__from_string,
    PasswordString_.c_tag: password_string__from_string,
    EncodedString_.c_tag: encoded_string__from_string,
    UsernameTokenType_.c_tag: username_token_type__from_string,
    BinarySecurityTokenType_.c_tag: binary_security_token_type__from_string,
    KeyIdentifierType_.c_tag: key_identifier_type__from_string,
    TUsage_.c_tag: t_usage__from_string,
    ReferenceType_.c_tag: reference_type__from_string,
    EmbeddedType_.c_tag: embedded_type__from_string,
    SecurityTokenReferenceType_.c_tag: security_token_reference_type__from_string,
    SecurityHeaderType_.c_tag: security_header_type__from_string,
    TransformationParametersType_.c_tag: transformation_parameters_type__from_string,
    UsernameToken.c_tag: username_token_from_string,
    BinarySecurityToken.c_tag: binary_security_token_from_string,
    Reference.c_tag: reference_from_string,
    Embedded.c_tag: embedded_from_string,
    KeyIdentifier.c_tag: key_identifier_from_string,
    SecurityTokenReference.c_tag: security_token_reference_from_string,
    Security.c_tag: security_from_string,
    TransformationParameters.c_tag: transformation_parameters_from_string,
    Password.c_tag: password_from_string,
    Nonce.c_tag: nonce_from_string,
    FaultcodeEnum_.c_tag: faultcode_enum__from_string,
    UsernameTokenType_Username.c_tag: username_token_type__username_from_string,
}

ELEMENT_BY_TAG = {
    "AttributedString": AttributedString_,
    "PasswordString": PasswordString_,
    "EncodedString": EncodedString_,
    "UsernameTokenType": UsernameTokenType_,
    "BinarySecurityTokenType": BinarySecurityTokenType_,
    "KeyIdentifierType": KeyIdentifierType_,
    "tUsage": TUsage_,
    "ReferenceType": ReferenceType_,
    "EmbeddedType": EmbeddedType_,
    "SecurityTokenReferenceType": SecurityTokenReferenceType_,
    "SecurityHeaderType": SecurityHeaderType_,
    "TransformationParametersType": TransformationParametersType_,
    "UsernameToken": UsernameToken,
    "BinarySecurityToken": BinarySecurityToken,
    "Reference": Reference,
    "Embedded": Embedded,
    "KeyIdentifier": KeyIdentifier,
    "SecurityTokenReference": SecurityTokenReference,
    "Security": Security,
    "TransformationParameters": TransformationParameters,
    "Password": Password,
    "Nonce": Nonce,
    "FaultcodeEnum": FaultcodeEnum_,
    "Username": UsernameTokenType_Username,
}


def factory(tag, **kwargs):
    return ELEMENT_BY_TAG[tag](**kwargs)



try:
    from importlib.metadata import version as _resolve_package_version
except ImportError:
    from importlib_metadata import version as _resolve_package_version


def _parse_version():
    value = _resolve_package_version("pysaml2")
    return value


version = _parse_version()
